import elasticsearch
import curator
import os
import json
import string, random, tempfile
import click
from click import testing as clicktest
from mock import patch, Mock

from . import CuratorTestCase

import logging
logger = logging.getLogger(__name__)

host, port = os.environ.get('TEST_ES_SERVER', 'localhost:9200').split(':')
port = int(port) if port else 9200

class TestGetClient(CuratorTestCase):
    def test_get_client_positive(self):
        client_args = {"host":host, "port":port}
        client = curator.get_client(**client_args)
        self.assertTrue(isinstance(client, elasticsearch.client.Elasticsearch))
    def test_get_client_negative_connection_fail(self):
        client_args = {"host":host, "port":54321}
        with self.assertRaises(SystemExit) as cm:
            curator.get_client(**client_args)
        self.assertEqual(cm.exception.code, 1)

class TestCLIIndexSelection(CuratorTestCase):
    def test_index_selection_only_timestamp_filter(self):
        self.create_indices(10)
        indices = curator.get_indices(self.client)
        # expected = sorted(indices, reverse=True)[:4]
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'close',
                        'indices',
                        '--timestring', '%Y.%m.%d',
                    ],
                    obj={"filters":[]})
        self.assertEqual(0, result.exit_code)
    def test_index_selection_no_filters(self):
        self.create_indices(1)
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'show',
                        'indices',
                    ],
                    obj={"filters":[]})
        self.assertEqual(1, result.exit_code)
    def test_index_selection_manual_selection_single(self):
        self.create_index('my_index')
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'show',
                        'indices',
                        '--index', 'my_index',
                    ],
                    obj={"filters":[]})
        self.assertEqual(['my_index'], result.output.splitlines()[:1])
    def test_index_selection_no_indices_test(self):
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'show',
                        'indices',
                        '--all-indices',
                    ],
                    obj={"filters":[]})
        self.assertEqual(1, result.exit_code)
    def test_index_selection_all_indices_single(self):
        self.create_index('my_index')
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'show',
                        'indices',
                        '--all-indices',
                    ],
                    obj={"filters":[]})
        self.assertEqual(['my_index'], result.output.splitlines()[:1])
    def test_index_selection_all_indices_exclude(self):
        self.create_index('my_index')
        self.create_index('your_index')
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'show',
                        'indices',
                        '--all-indices',
                        '--exclude', '^your.*$',
                    ],
                    obj={"filters":[]})
        self.assertEqual(['my_index'], result.output.splitlines()[:1])
    def test_index_selection_regex_match(self):
        self.create_index('my_index')
        self.create_index('your_index')
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'show',
                        'indices',
                        '--regex', '^my.*$',
                    ],
                    obj={"filters":[]})
        self.assertEqual(['my_index'], result.output.splitlines()[:1])
    def test_index_selection_all_indices_skip_non_exclude_filters(self):
        self.create_index('my_index')
        self.create_index('your_index')
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'show',
                        'indices',
                        '--all-indices',
                        '--suffix', 'index',
                    ],
                    obj={"filters":[]})
        self.assertEqual(['my_index', 'your_index'], result.output.splitlines()[:2])
    def test_cli_show_indices_if_dry_run(self):
        self.create_indices(10)
        indices = curator.get_indices(self.client)
        expected = sorted(indices, reverse=True)[:4]
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--dry-run',
                        '--host', host,
                        '--port', str(port),
                        'alias', '--name', 'dummy_alias',
                        'indices',
                        '--newer-than', '5',
                        '--timestring', '%Y.%m.%d',
                        '--time-unit', 'days'
                    ],
                    obj={"filters":[]})
        output = sorted(result.output.splitlines(), reverse=True)
        # I tried doing a nested, double list comprehension here.
        # It works in the interpreter, but not here for some reason.
        output = [ x.split(' ')[-1:] for x in output ]
        output = [ x[0] for x in output if x[0].startswith('logstash') ]
        self.assertEqual(expected, output)
    def test_cli_no_indices_after_filtering(self):
        self.create_indices(10)
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'show',
                        'indices',
                        '--exclude', 'log',
                    ],
                    obj={"filters":[]})
        self.assertEqual(0, result.exit_code)

class TestCLIAlias(CuratorTestCase):
    def test_alias_no_name_param(self):
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'alias',
                        'indices',
                        '--exclude', 'log',
                    ],
                    obj={"filters":[]})
        self.assertEqual(1, result.exit_code)
    def test_alias_all_indices_single(self):
        self.create_index('my_index')
        alias = 'testalias'
        self.create_index('dummy')
        self.client.indices.put_alias(index='dummy', name=alias)
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'alias', '--name', alias,
                        'indices',
                        '--all-indices',
                    ],
                    obj={"filters":[]})
        self.assertEquals(2, len(self.client.indices.get_alias(name=alias)))

class TestCLIAllocation(CuratorTestCase):
    def test_allocation_no_rule_param(self):
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'allocation',
                        'indices',
                        '--exclude', 'log',
                    ],
                    obj={"filters":[]})
        self.assertEqual(1, result.exit_code)
    def test_allocation_all_indices_single(self):
        self.create_index('my_index')
        key = 'foo'
        value = 'bar'
        rule = key + '=' + value
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'allocation', '--rule', rule,
                        'indices',
                        '--all-indices',
                    ],
                    obj={"filters":[]})
        self.assertEquals(
            value,
            self.client.indices.get_settings(index='my_index')['my_index']['settings']['index']['routing']['allocation']['require'][key]
        )

    def test_allocation_all_indices_include(self):
        self.create_index('my_index')
        key = 'foo'
        value = 'bar'
        rule = key + '=' + value
        allocation_type = 'include'
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'allocation', '--rule', rule,
                        '--type', allocation_type,
                        'indices',
                        '--all-indices',
                    ],
                    obj={"filters":[]})
        self.assertEquals(
            value,
            self.client.indices.get_settings(index='my_index')['my_index']['settings']['index']['routing']['allocation'][allocation_type][key]
        )

    def test_allocation_all_indices_exclude(self):
        self.create_index('my_index')
        key = 'foo'
        value = 'bar'
        rule = key + '=' + value
        allocation_type = 'exclude'
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'allocation', '--rule', rule,
                        '--type', allocation_type,
                        'indices',
                        '--all-indices',
                    ],
                    obj={"filters":[]})
        self.assertEquals(
            value,
            self.client.indices.get_settings(index='my_index')['my_index']['settings']['index']['routing']['allocation'][allocation_type][key]
        )

    def test_allocation_fail_on_bad_type(self):
        self.create_index('my_index')
        key = 'foo'
        value = 'bar'
        rule = key + '=' + value
        allocation_type = 'fail'
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'allocation', '--rule', rule,
                        '--type', allocation_type,
                        'indices',
                        '--all-indices',
                    ],
                    obj={"filters":[]})
        self.assertEqual(1, result.exit_code)

class TestCLIBloom(CuratorTestCase):
    def test_bloom_cli(self):
        self.create_indices(5)
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'bloom',
                        'indices',
                        '--all-indices',
                    ],
                    obj={"filters":[]})
        self.assertEqual(0, result.exit_code)

class TestCLIClose(CuratorTestCase):
    def test_close_cli(self):
        self.create_indices(5)
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'close',
                        'indices',
                        '--all-indices',
                    ],
                    obj={"filters":[]})
        self.assertEqual(0, result.exit_code)

class TestCLIDelete(CuratorTestCase):
    def test_delete_indices_skip_kibana(self):
        self.create_index('my_index')
        self.create_index('.kibana')
        self.create_index('kibana-int')
        self.create_index('.marvel-kibana')
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'delete',
                        'indices',
                        '--all-indices',
                    ],
                    obj={"filters":[]})
        l = curator.get_indices(self.client)
        self.assertEqual(sorted(['.kibana', '.marvel-kibana', 'kibana-int']), sorted(l))

    def test_delete_indices_dry_run(self):
        self.create_indices(9)
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--dry-run',
                        '--host', host,
                        '--port', str(port),
                        'delete',
                        'indices',
                        '--all-indices',
                    ],
                    obj={"filters":[]})
        l = curator.get_indices(self.client)
        self.assertEquals(9, len(l))
        output = sorted(result.output.splitlines(), reverse=True)
        # I tried doing a nested, double list comprehension here.
        # It works in the interpreter, but not here for some reason.
        output = [ x.split(' ')[-1:] for x in output ]
        output = [ x[0] for x in output if x[0].startswith('logstash') ]
        self.assertEqual(sorted(l, reverse=True), output)

    def test_delete_indices_by_space_dry_run(self):
        for i in range(1,10):
            self.client.create(
                index="index" + str(i), doc_type='log',
                body={'message':'TEST DOCUMENT'},
            )
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--dry-run',
                        '--host', host,
                        '--port', str(port),
                        'delete',
                        '--disk-space', '0.0000001',
                        'indices',
                        '--all-indices',
                    ],
                    obj={"filters":[]})
        l = curator.get_indices(self.client)
        self.assertEquals(9, len(l))
        output = sorted(result.output.splitlines(), reverse=True)
        # I tried doing a nested, double list comprehension here.
        # It works in the interpreter, but not here for some reason.
        output = [ x.split(' ')[-1:] for x in output ]
        output = [ x[0] for x in output if x[0].startswith('index') ]
        self.assertEqual(sorted(l, reverse=True), output)

    def test_delete_indices_huge_list(self):
        self.create_indices(365)
        pre = curator.get_indices(self.client)
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'delete',
                        'indices',
                        '--all-indices',
                        '--exclude', pre[0],
                    ],
                    obj={"filters":[]})
        post = curator.get_indices(self.client)
        self.assertEquals(1, len(post))

    def test_delete_indices_by_space_dry_run_huge_list(self):
        for i in range(100,150):
            self.client.create(
                index="superlongindexnamebyanystandardyouchoosethisissillyhowbigcanthisgetbeforeitbreaks" + str(i), doc_type='log',
                body={'message':'TEST DOCUMENT'},
            )
        l = curator.get_indices(self.client)
        self.assertEquals(50, len(l))
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--dry-run',
                        '--host', host,
                        '--port', str(port),
                        'delete',
                        '--disk-space', '0.0000001',
                        'indices',
                        '--all-indices',
                    ],
                    obj={"filters":[]})
        output = sorted(result.output.splitlines(), reverse=True)
        # # I tried doing a nested, double list comprehension here.
        # # It works in the interpreter, but not here for some reason.
        output = [ x.split(' ')[-1:] for x in output ]
        output = [ x[0] for x in output if x[0].startswith('superlongindexname') ]
        self.assertEqual(sorted(l, reverse=True), output)

class TestCLIOpen(CuratorTestCase):
    def test_open_cli(self):
        self.create_indices(5)
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'open',
                        'indices',
                        '--all-indices',
                    ],
                    obj={"filters":[]})
        self.assertEqual(0, result.exit_code)

class TestCLIOptimize(CuratorTestCase):
    def test_optimize_cli(self):
        self.create_indices(5)
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'optimize',
                        'indices',
                        '--all-indices',
                    ],
                    obj={"filters":[]})
        self.assertEqual(0, result.exit_code)
    def test_optimize_with_delay_cli(self):
        self.create_index("index_name")
        for i in ["1", "2", "3"]:
            self.client.create(
                index="index_name", doc_type='log',
                body={"doc" + i :'TEST DOCUMENT'},
            )
            # This should force each doc to be in its own segment.
            self.client.indices.flush(index="index_name", force=True)

        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'optimize',
                        '--delay', 1,
                        'indices',
                        '--all-indices',
                    ],
                    obj={"filters":[]})
        self.assertEqual(0, result.exit_code)

class TestCLIReplicas(CuratorTestCase):
    def test_replicas_no_count_param(self):
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'replicas',
                        'indices',
                        '--exclude', 'log',
                    ],
                    obj={"filters":[]})
        self.assertEqual(1, result.exit_code)
    def test_replicas_cli(self):
        self.create_indices(5)
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'replicas', '--count', '2',
                        'indices',
                        '--all-indices',
                    ],
                    obj={"filters":[]})
        self.assertEqual(0, result.exit_code)

class TestCLISeal(CuratorTestCase):
    def test_cli_seal_indices(self):
        self.create_indices(10)
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'seal',
                        'indices',
                        '--newer-than', '5',
                        '--timestring', '%Y.%m.%d',
                        '--time-unit', 'days'
                    ],
                    obj={"filters":[]})
        self.assertEqual(0, result.exit_code)

class TestCLIShow(CuratorTestCase):
    def test_cli_show_indices(self):
        self.create_indices(10)
        indices = curator.get_indices(self.client)
        expected = sorted(indices, reverse=True)[:4]
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'show',
                        'indices',
                        '--newer-than', '5',
                        '--timestring', '%Y.%m.%d',
                        '--time-unit', 'days'
                    ],
                    obj={"filters":[]})
        output = sorted(result.output.splitlines(), reverse=True)[:4]
        self.assertEqual(expected, output)
    def test_cli_show_indices_older_than_zero(self):
        self.create_indices(10)
        indices = curator.get_indices(self.client)
        expected = sorted(indices, reverse=True)
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'show',
                        'indices',
                        '--older-than', '0',
                        '--timestring', '%Y.%m.%d',
                        '--time-unit', 'days'
                    ],
                    obj={"filters":[]})
        output = sorted(result.output.splitlines(), reverse=True)
        self.assertEqual(expected, output)

class TestCLISnapshot(CuratorTestCase):
    def test_snapshot_no_repository_param(self):
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'snapshot',
                        'indices',
                        '--exclude', 'log',
                    ],
                    obj={"filters":[]})
        self.assertEqual(1, result.exit_code)
    def test_cli_snapshot_indices(self):
        self.create_indices(5)
        self.create_repository()
        snap_name = 'snapshot1'
        indices = curator.get_indices(self.client)
        expected = sorted(indices, reverse=True)[:4]
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'snapshot',
                        '--repository', self.args['repository'],
                        '--name', snap_name,
                        'indices',
                        '--all-indices',
                    ],
                    obj={"filters":[]})
        snapshot = curator.get_snapshot(
                    self.client, self.args['repository'], '_all'
                   )
        self.assertEqual(1, len(snapshot['snapshots']))
        self.assertEqual(snap_name, snapshot['snapshots'][0]['snapshot'])
    def test_cli_snapshot_huge_list(self):
        self.create_indices(200)
        self.create_repository()
        snap_name = 'snapshot1'
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        '--timeout', 600,
                        'snapshot',
                        '--repository', self.args['repository'],
                        '--name', snap_name,
                        'indices',
                        '--all-indices',
                    ],
                    obj={"filters":[]})
        snapshot = curator.get_snapshot(
                    self.client, self.args['repository'], '_all'
                   )
        self.assertEqual(1, len(snapshot['snapshots']))
        self.assertEqual(snap_name, snapshot['snapshots'][0]['snapshot'])
        self.assertEqual(200, len(snapshot['snapshots'][0]['indices']))


class TestCLISnapshotSelection(CuratorTestCase):
    def test_show_no_repository_param(self):
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'show',
                        'snapshots',
                        '--exclude', 'log',
                    ],
                    obj={"filters":[]})
        self.assertEqual(1, result.exit_code)
    def test_snapshot_selection_show_all_snapshots(self):
        index_name = 'index1'
        snap_name = 'snapshot1'
        self.create_index(index_name)
        self.create_repository()
        self.client.create(
            index=index_name, doc_type='log', body={'message':'TEST DOCUMENT'}
            )
        curator.create_snapshot(
            self.client, name=snap_name, indices=index_name,
            repository=self.args['repository']
            )
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'show',
                        'snapshots',
                        '--repository', self.args['repository'],
                        '--all-snapshots',
                    ],
                    obj={"filters":[]})
        self.assertEqual([snap_name], result.output.splitlines()[:1])
    def test_snapshot_selection_show_all_snapshots_with_exclude(self):
        self.create_repository()
        for i in ["1", "2"]:
            self.create_index("index" + i)
            self.client.create(
                index="index" + i, doc_type='log',
                body={'message':'TEST DOCUMENT'},
            )
            curator.create_snapshot(
                self.client, name="snapshot" + i, indices="index" + i,
                repository=self.args['repository']
            )
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'show',
                        'snapshots',
                        '--repository', self.args['repository'],
                        '--all-snapshots',
                        '--exclude', '2',
                    ],
                    obj={"filters":[]})
        self.assertEqual(['snapshot1'], result.output.splitlines()[:1])
    def test_snapshot_selection_no_snapshots_found(self):
        self.create_repository()
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'show',
                        'snapshots',
                        '--repository', self.args['repository'],
                        '--all-snapshots',
                    ],
                    obj={"filters":[]})
        self.assertEqual(0, result.exit_code)
    def test_snapshot_selection_show_filtered(self):
        self.create_repository()
        for i in ["1", "2", "3"]:
            self.create_index("index" + i)
            self.client.create(
                index="index" + i, doc_type='log',
                body={'message':'TEST DOCUMENT'},
            )
            curator.create_snapshot(
                self.client, name="snapshot" + i, indices="index" + i,
                repository=self.args['repository']
            )
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'show',
                        'snapshots',
                        '--repository', self.args['repository'],
                        '--prefix', 'snap',
                        '--suffix', '1',
                    ],
                    obj={"filters":[]})
        self.assertEqual(['snapshot1'], result.output.splitlines()[:1])
    def test_snapshot_selection_show_all_snapshots_with_exclude_and_other(self):
        self.create_repository()
        for i in ["1", "2"]:
            self.create_index("index" + i)
            self.client.create(
                index="index" + i, doc_type='log',
                body={'message':'TEST DOCUMENT'},
            )
            curator.create_snapshot(
                self.client, name="snapshot" + i, indices="index" + i,
                repository=self.args['repository']
            )
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'show',
                        'snapshots',
                        '--repository', self.args['repository'],
                        '--all-snapshots',
                        '--exclude', '2',
                        '--prefix', 'missing',
                        '--suffix', 'also_missing',
                    ],
                    obj={"filters":[]})
        self.assertEqual(['snapshot1'], result.output.splitlines()[:1])
    def test_snapshot_selection_delete_all_snapshots_with_dry_run(self):
        self.create_repository()
        for i in ["1", "2"]:
            self.create_index("index" + i)
            self.client.create(
                index="index" + i, doc_type='log',
                body={'message':'TEST DOCUMENT'},
            )
            curator.create_snapshot(
                self.client, name="snapshot" + i, indices="index" + i,
                repository=self.args['repository']
            )
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--dry-run',
                        '--host', host,
                        '--port', str(port),
                        'delete',
                        'snapshots',
                        '--repository', self.args['repository'],
                        '--all-snapshots',
                        '--exclude', '2',
                    ],
                    obj={"filters":[]})
        output = sorted(result.output.splitlines(), reverse=True)[:4]
        # I tried doing a nested, double list comprehension here.
        # It works in the interpreter, but not here for some reason.
        output = [ x.split(' ')[-1:] for x in output ]
        output = [ x[0] for x in output if x[0].startswith('snapshot1') ]
        self.assertEqual(['snapshot1'], output)
    def test_snapshot_selection_delete_snapshot(self):
        self.create_repository()
        for i in ["1", "2"]:
            self.create_index("index" + i)
            self.client.create(
                index="index" + i, doc_type='log',
                body={'message':'TEST DOCUMENT'},
            )
            curator.create_snapshot(
                self.client, name="snapshot" + i, indices="index" + i,
                repository=self.args['repository']
            )
        result = curator.get_snapshot(self.client, self.args['repository'], '_all')
        self.assertEqual(2, len(result['snapshots']))
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'delete',
                        'snapshots',
                        '--repository', self.args['repository'],
                        '--exclude', '2',
                    ],
                    obj={"filters":[]})
        result = curator.get_snapshot(self.client, self.args['repository'], '_all')
        self.assertEqual(1, len(result['snapshots']))
        self.assertEqual('snapshot2', result['snapshots'][0]['snapshot'])
    def test_snapshot_selection_all_filtered_fail(self):
        self.create_repository()
        for i in ["1", "2", "3"]:
            self.create_index("index" + i)
            self.client.create(
                index="index" + i, doc_type='log',
                body={'message':'TEST DOCUMENT'},
            )
            curator.create_snapshot(
                self.client, name="snapshot" + i, indices="index" + i,
                repository=self.args['repository']
            )
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'show',
                        'snapshots',
                        '--repository', self.args['repository'],
                        '--prefix', 'no_match',
                    ],
                    obj={"filters":[]})
        self.assertEqual(0, result.exit_code)

class TestCLILogging(CuratorTestCase):
    def test_logging_with_debug_flag(self):
        self.create_indices(10)
        indices = curator.get_indices(self.client)
        expected = sorted(indices, reverse=True)[:4]
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--debug',
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'show',
                        'indices',
                        '--newer-than', '5',
                        '--timestring', '%Y.%m.%d',
                        '--time-unit', 'days'
                    ],
                    obj={"filters":[]})
        output = sorted(result.output.splitlines(), reverse=True)[:4]
        self.assertEqual(expected, output)

    def test_logging_with_bad_loglevel_flag(self):
        self.create_indices(1)
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--loglevel', 'FOO',
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'show',
                        'indices',
                        '--newer-than', '5',
                        '--timestring', '%Y.%m.%d',
                        '--time-unit', 'days'
                    ],
                    obj={"filters":[]})
        self.assertEqual('Invalid log level: FOO', str(result.exception))

    def test_logging_with_logformat_logstash_flag(self):
        self.create_indices(10)
        indices = curator.get_indices(self.client)
        expected = sorted(indices, reverse=True)[:4]
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logformat', 'logstash',
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'show',
                        'indices',
                        '--newer-than', '5',
                        '--timestring', '%Y.%m.%d',
                        '--time-unit', 'days'
                    ],
                    obj={"filters":[]})
        output = sorted(result.output.splitlines(), reverse=True)[:4]
        self.assertEqual(expected, output)

class TestCLIOptions(CuratorTestCase):
    def test_logstash_formatting(self):
        dirname = ''.join(random.choice(string.ascii_uppercase + string.digits) for _ in range(8))
        logfile = tempfile.mkdtemp(suffix=dirname) + 'logfile'
        self.create_indices(1)
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logformat', 'logstash',
                        '--debug',
                        '--host', host,
                        '--port', str(port),
                        'show',
                        'indices', '--all-indices'
                    ],
                    obj={"filters":[]})
        d = json.loads(result.output.splitlines()[:1][0])
        keys = sorted(list(d.keys()))
        self.assertEqual(['@timestamp','function','linenum','loglevel','message','name'], keys)



from datetime import datetime, timedelta
from unittest import TestCase
from mock import Mock
import sys
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

import elasticsearch
from curator import api as curator

named_index    = 'index_name'
named_indices  = [ "index1", "index2" ]
open_index     = {'metadata': {'indices' : { named_index : {'state' : 'open'}}}}
closed_index   = {'metadata': {'indices' : { named_index : {'state' : 'close'}}}}
open_indices   = { 'metadata': { 'indices' : { 'index1' : { 'state' : 'open' },
                                               'index2' : { 'state' : 'open' }}}}
closed_indices = { 'metadata': { 'indices' : { 'index1' : { 'state' : 'close' },
                                               'index2' : { 'state' : 'close' }}}}
fake_fail      = Exception('Simulated Failure')
named_alias    = 'alias_name'
allocation_in  = {named_index: {'settings': {'index': {'routing': {'allocation': {'require': {'foo': 'bar'}}}}}}}
allocation_out = {named_index: {'settings': {'index': {'routing': {'allocation': {'require': {'not': 'foo'}}}}}}}
alias_retval   = { "pre_aliased_index": { "aliases" : { named_alias : { }}}}
aliases_retval = {
    "index1": { "aliases" : { named_alias : { } } },
    "index2": { "aliases" : { named_alias : { } } },
    }
indices_space  = { 'indices' : {
        'index1' : { 'index' : { 'primary_size_in_bytes': 1083741824 }},
        'index2' : { 'index' : { 'primary_size_in_bytes': 1083741824 }}}}
shards         = { 'indices': { named_index: { 'shards': {
        '0': [ { 'num_search_segments' : 15 }, { 'num_search_segments' : 21 } ],
        '1': [ { 'num_search_segments' : 19 }, { 'num_search_segments' : 16 } ] }}}}
optimize_tuple = (4, 71)
snap_name      = 'snap_name'
repo_name      = 'repo_name'
snap_running   = { 'snapshots': ['running'] }
nosnap_running = { 'snapshots': [] }
snapshot       = { 'snapshots': [
                    {
                        'duration_in_millis': 60000, 'start_time': '2015-01-01T00:00:00.000Z',
                        'shards': {'successful': 4, 'failed': 0, 'total': 4},
                        'end_time_in_millis': 0, 'state': 'SUCCESS',
                        'snapshot': snap_name, 'end_time': '2015-01-01T00:00:01.000Z',
                        'indices': named_indices,
                        'failures': [], 'start_time_in_millis': 0
                    }]}
partial        = { 'snapshots': [
                    {
                        'duration_in_millis': 60000, 'start_time': '2015-01-01T00:00:00.000Z',
                        'shards': {'successful': 4, 'failed': 0, 'total': 4},
                        'end_time_in_millis': 0, 'state': 'PARTIAL',
                        'snapshot': snap_name, 'end_time': '2015-01-01T00:00:01.000Z',
                        'indices': named_indices,
                        'failures': [], 'start_time_in_millis': 0
                    }]}
snapshots       = { 'snapshots': [
                    {
                        'duration_in_millis': 60000, 'start_time': '2015-01-01T00:00:00.000Z',
                        'shards': {'successful': 4, 'failed': 0, 'total': 4},
                        'end_time_in_millis': 0, 'state': 'SUCCESS',
                        'snapshot': snap_name, 'end_time': '2015-01-01T00:00:01.000Z',
                        'indices': named_indices,
                        'failures': [], 'start_time_in_millis': 0
                    },
                    {
                        'duration_in_millis': 60000, 'start_time': '2015-01-01T00:00:02.000Z',
                        'shards': {'successful': 4, 'failed': 0, 'total': 4},
                        'end_time_in_millis': 0, 'state': 'SUCCESS',
                        'snapshot': 'snapshot2', 'end_time': '2015-01-01T00:00:03.000Z',
                        'indices': named_indices,
                        'failures': [], 'start_time_in_millis': 0
                    }]}
snap_body_all   = {
                    "ignore_unavailable": False,
                    "include_global_state": True,
                    "partial": False,
                    "indices" : "_all"
                  }
snap_body       = {
                    "ignore_unavailable": False,
                    "include_global_state": True,
                    "partial": False,
                    "indices" : "index1,index2"
                  }
verified_nodes  = {'nodes': {'nodeid1': {'name': 'node1'}, 'nodeid2': {'name': 'node2'}}}
synced_pass     = {
                    "_shards":{"total":1,"successful":1,"failed":0},
                    "index_name":{
                        "total":1,"successful":1,"failed":0,
                        "failures":[],
                    }
                  }
synced_fail     = {
                    "_shards":{"total":1,"successful":0,"failed":1},
                    "index_name":{
                        "total":1,"successful":0,"failed":1,
                        "failures":[
                            {"shard":0,"reason":"pending operations","routing":{"state":"STARTED","primary":True,"node":"nodeid1","relocating_node":None,"shard":0,"index":"index_name"}},
                        ]
                    }
                  }
sync_conflict   = elasticsearch.ConflictError(409, u'{"_shards":{"total":1,"successful":0,"failed":1},"index_name":{"total":1,"successful":0,"failed":1,"failures":[{"shard":0,"reason":"pending operations","routing":{"state":"STARTED","primary":true,"node":"nodeid1","relocating_node":null,"shard":0,"index":"index_name"}}]}})', synced_fail)
synced_fails    = {
                    "_shards":{"total":2,"successful":1,"failed":1},
                    "index1":{
                        "total":1,"successful":0,"failed":1,
                        "failures":[
                            {"shard":0,"reason":"pending operations","routing":{"state":"STARTED","primary":True,"node":"nodeid1","relocating_node":None,"shard":0,"index":"index_name"}},
                        ]
                    },
                    "index2":{
                        "total":1,"successful":1,"failed":0,
                        "failures":[]
                    },
                  }

class TestAlias(TestCase):
    def test_add_to_alias_bad_csv(self):
        client = Mock()
        c = "a,b,c,d"
        self.assertFalse(curator.add_to_alias(client, c))
    def test_add_to_alias_no_alias_arg(self):
        client = Mock()
        self.assertFalse(curator.add_to_alias(client, named_index))
    def test_add_to_alias_alias_not_found(self):
        client = Mock()
        client.indices.exists_alias.return_value = False
        client.cluster.state.return_value = open_index
        self.assertTrue(curator.add_to_alias(client, named_index, alias=named_alias))
    def test_add_to_alias_exception_test(self):
        client = Mock()
        client.indices.get_alias.return_value = alias_retval
        client.indices.exists_alias.return_value = True
        client.cluster.state.return_value = open_index
        client.indices.update_aliases.side_effect = fake_fail
        self.assertFalse(curator.add_to_alias(client, named_index, alias=named_alias))
    def test_remove_from_alias_bad_csv(self):
        client = Mock()
        self.assertFalse(curator.remove_from_alias(client, "a,b,c,d"))
    def test_remove_from_alias_no_alias_arg(self):
        client = Mock()
        self.assertFalse(curator.remove_from_alias(client, named_index))
    def test_remove_from_alias_alias_not_found(self):
        client = Mock()
        client.indices.exists_alias.return_value = False
        self.assertFalse(curator.remove_from_alias(client, named_index, alias=named_alias))
    def test_remove_from_alias_exception_raised(self):
        client = Mock()
        client.indices.exists_alias.return_value = True
        client.indices.get_alias.return_value = aliases_retval
        client.indices.update_aliases.side_effect = fake_fail
        self.assertRaises(Exception, curator.remove_from_alias(client, "index1", alias=named_alias))
    def test_remove_from_alias_exception_return_false(self):
        client = Mock()
        client.indices.exists_alias.return_value = True
        client.indices.get_alias.return_value = aliases_retval
        client.indices.update_aliases.side_effect = fake_fail
        self.assertFalse(curator.remove_from_alias(client, "index1", alias=named_alias))
    def test_remove_from_alias_index_not_found_in_alias(self):
        client = Mock()
        client.indices.exists_alias.return_value = True
        client.indices.get_alias.return_value = aliases_retval
        self.assertFalse(curator.remove_from_alias(client, "foo", alias=named_alias))

class TestAllocate(TestCase):
    def test_apply_allocation_rule_param_check(self):
        client = Mock()
        # Testing for the omission of the rule param
        self.assertFalse(curator.apply_allocation_rule(client, named_indices))
    def test_allocation_rule_positive(self):
        client = Mock()
        client.indices.get_settings.return_value = allocation_out
        client.cluster.state.return_value = open_index
        client.indices.put_settings.return_value = None
        self.assertTrue(curator.apply_allocation_rule(client, named_index, rule="foo=bar"))
    def test_apply_allocation_rule_negative(self):
        client = Mock()
        client.cluster.state.return_value = open_index
        client.indices.get_settings.return_value = allocation_in
        client.indices.put_settings.return_value = None
        self.assertFalse(curator.apply_allocation_rule(client, named_index, rule="foo=bar"))
    def test_apply_allocation_rule_empty_list(self):
        client = Mock()
        self.assertFalse(curator.apply_allocation_rule(client, [], rule="foo=bar"))
    def test_allocation_positive(self):
        client = Mock()
        client.cluster.state.return_value = open_index
        client.indices.get_settings.return_value = allocation_out
        client.indices.put_settings.return_value = None
        self.assertTrue(curator.allocation(client, named_index, rule="foo=bar"))
    def test_allocation_negative_exception(self):
        client = Mock()
        client.cluster.state.return_value = open_index
        client.indices.get_settings.return_value = allocation_out
        client.indices.put_settings.side_effect = fake_fail
        self.assertFalse(curator.allocation(client, named_index, rule="foo=bar"))
    def test_allocation_wrong_type_param(self):
        client = Mock()
        client.cluster.state.return_value = open_index
        client.indices.get_settings.return_value = allocation_out
        client.indices.put_settings.return_value = None
        self.assertFalse(curator.allocation(client, named_index, rule="foo=bar", allocation_type="wrong_type"))
    def test_allocation_good_type_param(self):
        client = Mock()
        client.cluster.state.return_value = open_index
        client.indices.get_settings.return_value = allocation_out
        client.indices.put_settings.return_value = None
        self.assertTrue(curator.allocation(client, named_index, rule="foo=bar", allocation_type="require"))   
        self.assertTrue(curator.allocation(client, named_index, rule="foo=bar", allocation_type="include"))   
        self.assertTrue(curator.allocation(client, named_index, rule="foo=bar", allocation_type="exclude"))   

class TestBloom(TestCase):
    def test_disable_bloom_no_more_bloom_positive(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '1.4.4'} }
        self.assertTrue(curator.disable_bloom_filter(client, named_index))
    def test_disable_bloom_no_more_bloom_positive(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '1.3.4'} }
        client.cluster.state.return_value = open_index
        client.indices.put_settings.return_value = None
        self.assertTrue(curator.disable_bloom_filter(client, named_index))
    def test_disable_bloom_exception_test(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '1.3.4'} }
        client.cluster.state.return_value = open_index
        client.indices.put_settings.side_effect = fake_fail
        self.assertRaises(Exception, curator.disable_bloom_filter(client, named_index))
    def test_disable_bloom_with_delay_positive(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '1.3.4'} }
        client.cluster.state.return_value = open_indices
        self.assertTrue(curator.disable_bloom_filter(
            client, named_indices, delay=1
            ))
    def test_disable_bloom_with_delay_negative(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '1.3.4'} }
        client.cluster.state.return_value = open_indices
        client.indices.put_settings.side_effect = fake_fail
        self.assertFalse(curator.disable_bloom_filter(
            client, named_indices, delay=1
            ))
    def test_bloom_full_positive(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '1.3.4'} }
        client.cluster.state.return_value = open_index
        client.indices.put_settings.return_value = None
        self.assertTrue(curator.bloom(client, named_index))
    def test_bloom_full_negative(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '1.3.4'} }
        client.cluster.state.return_value = open_index
        client.indices.put_settings.side_effect = fake_fail
        self.assertFalse(curator.bloom(client, named_index))

class TestClose(TestCase):
    def test_close_indices_positive_presyncflush(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '1.3.4'} }
        client.indices.flush.return_value = None
        client.indices.close.return_value = None
        self.assertTrue(curator.close_indices(client, named_index))
    def test_close_indices_negative_presyncflush(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '1.3.4'} }
        client.indices.flush.return_value = None
        client.indices.close.side_effect = fake_fail
        client.indices.close.return_value = None
        self.assertFalse(curator.close_indices(client, named_index))
    def test_full_close_positive_presyncflush(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '1.3.4'} }
        client.indices.flush.return_value = None
        client.indices.close.return_value = None
        self.assertTrue(curator.close(client, named_index))
    def test_full_close_negative_presyncflush(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '1.3.4'} }
        client.indices.flush.return_value = None
        client.indices.close.side_effect = fake_fail
        client.indices.close.return_value = None
        self.assertFalse(curator.close(client, named_index))
    def test_close_indices_positive(self):
        client = Mock()
        client.cluster.state.return_value = open_index
        client.indices.flush_synced.return_value = synced_pass
        client.info.return_value = {'version': {'number': '1.6.0'} }
        client.indices.close.return_value = None
        self.assertTrue(curator.close_indices(client, named_index))
    def test_close_indices_negative(self):
        client = Mock()
        client.cluster.state.return_value = open_index
        client.indices.flush_synced.return_value = synced_fail
        client.info.return_value = {'version': {'number': '1.6.0'} }
        client.indices.close.side_effect = fake_fail
        client.indices.close.return_value = None
        self.assertFalse(curator.close_indices(client, named_index))
    def test_full_close_positive(self):
        client = Mock()
        client.cluster.state.return_value = open_index
        client.indices.flush_synced.return_value = synced_pass
        client.info.return_value = {'version': {'number': '1.6.0'} }
        client.indices.close.return_value = None
        self.assertTrue(curator.close(client, named_index))
    def test_full_close_negative(self):
        client = Mock()
        client.cluster.state.return_value = open_index
        client.indices.flush_synced.return_value = synced_fail
        client.info.return_value = {'version': {'number': '1.6.0'} }
        client.indices.close.side_effect = fake_fail
        client.indices.close.return_value = None
        self.assertFalse(curator.close(client, named_index))

class TestDelete(TestCase):
    def test_delete_indices_positive(self):
        client = Mock()
        client.indices.delete.return_value = None
        self.assertTrue(curator.delete_indices(client, named_indices))
    def test_delete_indices_negative(self):
        client = Mock()
        client.indices.delete.side_effect = fake_fail
        self.assertFalse(curator.delete_indices(client, named_indices))

    # This test needs to be able to have get_settings return two different
    # values on subsequent calls.  I don't know how to do that, if it can
    # be done.  Integration testing can cover testing this method, though.
    #
    # def test_full_delete_positive(self):
    #     client = Mock()
    #     client.indices.delete.return_value = None
    #     client.indices.get_settings.return_value = named_indices
    #     self.assertTrue(curator.delete(client, named_indices))
    def test_full_delete_negative(self):
        client = Mock()
        client.indices.delete.return_value = None
        client.indices.get_settings.return_value = named_indices
        self.assertFalse(curator.delete(client, named_indices))
    def test_full_delete_exception(self):
        client = Mock()
        client.indices.delete.side_effect = fake_fail
        client.indices.get_settings.return_value = named_indices
        self.assertFalse(curator.delete(client, named_indices))

class TestOpen(TestCase):
    def test_opener_positive(self):
        client = Mock()
        client.indices.open.return_value = None
        self.assertTrue(curator.open_indices(client, named_indices))
    def test_opener_negative(self):
        client = Mock()
        client.indices.open.side_effect = fake_fail
        self.assertFalse(curator.open_indices(client, named_indices))
    def test_full_opener_positive(self):
        client = Mock()
        client.indices.open.return_value = None
        self.assertTrue(curator.opener(client, named_indices))
    def test_full_opener_negative(self):
        client = Mock()
        client.indices.open.side_effect = fake_fail
        self.assertFalse(curator.opener(client, named_indices))

class TestOptimize(TestCase):
    def test_optimize_index_positive(self):
        client = Mock()
        client.indices.segments.return_value = shards
        client.cluster.state.return_value = open_index
        client.indices.optimize.return_value = None
        self.assertTrue(curator.optimize_index(client, named_index, max_num_segments=2))
    def test_optimize_index_negative(self):
        client = Mock()
        client.indices.segments.return_value = shards
        client.cluster.state.return_value = open_index
        client.indices.optimize.side_effect = fake_fail
        self.assertFalse(curator.optimize_index(client, named_index, max_num_segments=2))
    def test_optimize_positive(self):
        client = Mock()
        client.indices.segments.return_value = shards
        client.cluster.state.return_value = open_index
        client.indices.optimize.return_value = None
        self.assertTrue(curator.optimize(client, named_index, max_num_segments=2))
    def test_optimize_negative(self):
        client = Mock()
        client.indices.segments.return_value = shards
        client.cluster.state.return_value = open_index
        client.indices.optimize.side_effect = fake_fail
        self.assertFalse(curator.optimize(client, named_index, max_num_segments=2))

class TestReplicas(TestCase):
    def test_change_replicas_param_check(self):
        client = Mock()
        # Testing for the omission of the replicas param
        self.assertFalse(curator.change_replicas(client, named_indices))
    def test_change_replicas_positive(self):
        client = Mock()
        client.cluster.state.return_value = open_indices
        client.indices.put_settings.return_value = None
        self.assertTrue(curator.change_replicas(client, named_indices, replicas=0))
    def test_change_replicas_negative(self):
        client = Mock()
        client.cluster.state.return_value = open_indices
        client.indices.put_settings.side_effect = fake_fail
        self.assertFalse(curator.change_replicas(client, named_indices, replicas=0))
    def test_replicas_positive(self):
        client = Mock()
        client.cluster.state.return_value = open_indices
        client.indices.put_settings.return_value = None
        self.assertTrue(curator.replicas(client, named_indices, replicas=0))
    def test_replicas_negative(self):
        client = Mock()
        client.cluster.state.return_value = open_indices
        client.indices.put_settings.side_effect = fake_fail
        self.assertFalse(curator.replicas(client, named_indices, replicas=0))

class TestSeal(TestCase):
    # The seal_indices method pretty much always returns True, requiring log
    # viewing to ascertain if one or more indices failed to seal.
    def test_seal_indices_good_version(self):
        client = Mock()
        client.cluster.state.return_value = open_index
        client.indices.flush_synced.return_value = synced_pass
        client.info.return_value = {'version': {'number': '1.6.0'} }
        self.assertTrue(curator.seal_indices(client, named_index))
    def test_seal_indices_bad_version(self):
        client = Mock()
        client.cluster.state.return_value = open_index
        client.indices.flush_synced.return_value = synced_pass
        client.info.return_value = {'version': {'number': '1.3.4'} }
        self.assertTrue(curator.seal_indices(client, named_index))
    def test_seal_indices_conflicterror(self):
        client = Mock()
        client.cluster.state.return_value = open_index
        client.indices.flush_synced.return_value = synced_fail
        client.indices.flush_synced.side_effect = sync_conflict
        client.info.return_value = {'version': {'number': '1.6.0'} }
        self.assertTrue(curator.seal_indices(client, named_index))
    def test_seal_indices_onepass_onefail(self):
        client = Mock()
        client.cluster.state.return_value = open_index
        client.indices.flush_synced.return_value = synced_fails
        client.info.return_value = {'version': {'number': '1.6.0'} }
        self.assertTrue(curator.seal_indices(client, named_index))
    def test_seal_indices_attribute_exception(self):
        client = Mock()
        client.cluster.state.return_value = open_index
        client.indices.flush_synced.return_value = synced_fail
        client.indices.flush_synced.side_effect = fake_fail
        client.info.return_value = {'version': {'number': '1.6.0'} }
        self.assertTrue(curator.seal_indices(client, named_index))

class TestShow(TestCase):
    def setUp(self):
        self.held, sys.stdout = sys.stdout, StringIO()
    def test_show_positive(self):
        client = Mock()
        client.cluster.state.return_value = open_index
        curator.show(client, named_index)
        self.assertEqual(sys.stdout.getvalue(),'index_name\n')
    def test_show_positive_list(self):
        client = Mock()
        client.cluster.state.return_value = open_indices
        curator.show(client, named_indices)
        self.assertEqual(sys.stdout.getvalue(),'index1\nindex2\n')

class TestSnapshot(TestCase):
    def test_create_snapshot_missing_arg_repository(self):
        client = Mock()
        self.assertFalse(curator.create_snapshot(client, name=snap_name))
    def test_create_snapshot_in_progress(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '1.4.4'} }
        client.snapshot.status.return_value = snap_running
        self.assertFalse(curator.create_snapshot(client, indices=[], repository=repo_name, name=snap_name))
    def test_create_snapshot_in_progress_old_version(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '1.0.3'} }
        client.snapshot.status.return_value = snap_running
        self.assertFalse(curator.create_snapshot(client, indices=[], repository=repo_name, name=snap_name))
    def test_create_snapshot_empty_arg_indices(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '1.4.4'} }
        client.snapshot.status.return_value = nosnap_running
        self.assertFalse(curator.create_snapshot(client, indices=[], repository=repo_name, name=snap_name))
    def test_create_snapshot_verify_nodes_positive(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '1.4.4'} }
        client.cluster.state.return_value = open_indices
        client.snapshot.get.return_value = snapshots
        client.snapshot.verify_repository.return_value = verified_nodes
        client.snapshot.status.return_value = nosnap_running
        self.assertTrue(
            curator.create_snapshot(
                client,
                indices=named_indices,
                repository=repo_name,
                name='not_snap_name'
            )
        )
    def test_create_snapshot_verify_nodes_negative(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '1.4.4'} }
        client.cluster.state.return_value = open_indices
        client.snapshot.get.return_value = snapshots
        client.snapshot.verify_repository.return_value = verified_nodes
        client.snapshot.verify_repository.side_effect = fake_fail
        client.snapshot.status.return_value = nosnap_running
        self.assertFalse(
            curator.create_snapshot(
                client,
                indices=named_indices,
                repository=repo_name,
                name='not_snap_name'
            )
        )
    def test_create_snapshot_name_collision(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '1.4.4'} }
        client.cluster.state.return_value = open_indices
        client.snapshot.get.return_value = snapshots
        client.snapshot.verify_repository.return_value = verified_nodes
        client.snapshot.status.return_value = nosnap_running
        self.assertFalse(
            self.assertFalse(
                curator.create_snapshot(
                    client,
                    indices=named_indices,
                    repository=repo_name,
                    name=snap_name
                )
            )
        )
    def test_create_snapshot_exception(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '1.4.4'} }
        client.cluster.state.return_value = open_indices
        client.snapshot.get.return_value = snapshots
        client.snapshot.verify_repository.return_value = verified_nodes
        client.snapshot.create.side_effect = elasticsearch.TransportError
        client.snapshot.status.return_value = nosnap_running
        self.assertFalse(
            curator.create_snapshot(
                client,
                indices=named_indices,
                repository=repo_name,
                name='not_snap_name'
            )
        )
    def test_create_snapshot_no_wait_for_completion(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '1.4.4'} }
        client.cluster.state.return_value = open_indices
        client.snapshot.get.return_value = snapshot
        client.snapshot.verify_repository.return_value = verified_nodes
        client.snapshot.status.return_value = nosnap_running
        self.assertTrue(
            curator.create_snapshot(
                client,
                indices=named_indices,
                repository=repo_name,
                wait_for_completion=False,
                name='not_snap_name'
            )
        )
    def test_create_snapshot_incomplete(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '1.4.4'} }
        client.cluster.state.return_value = open_indices
        client.snapshot.get.return_value = partial
        client.snapshot.verify_repository.return_value = verified_nodes
        client.snapshot.status.return_value = nosnap_running
        self.assertFalse(
            curator.create_snapshot(
                client,
                indices=named_indices,
                repository=repo_name,
                name='not_snap_name'
            )
        )

class TestDeleteSnapshot(TestCase):
    def test_delete_snapshot_missing_arg_repository(self):
        client = Mock()
        self.assertFalse(curator.delete_snapshot(client, snapshot=snap_name))
    def test_delete_snapshot_missing_arg_name(self):
        client = Mock()
        self.assertFalse(curator.delete_snapshot(client, repository=repo_name))
    def test_delete_snapshot_snapshot_is_list(self):
        client = Mock()
        self.assertFalse(curator.delete_snapshot(client, repository=repo_name, snapshot=['snap1', 'snap2']))
    def test_delete_snapshot_positive(self):
        client = Mock()
        client.snapshot.delete.return_value = None
        self.assertTrue(curator.delete_snapshot(client, repository=repo_name, snapshot=snap_name))
    def test_delete_snapshot_negative(self):
        client = Mock()
        client.snapshot.delete.side_effect = elasticsearch.RequestError
        self.assertFalse(curator.delete_snapshot(client, repository=repo_name, snapshot=snap_name))



import elasticsearch
import click
from .index_selection import *

import logging
logger = logging.getLogger(__name__)

@cli.group('allocation')
@click.option('--rule', show_default=True, expose_value=True, type=str,
            help='Routing allocation rule to apply, e.g. tag=ssd')
@click.option('--type', default='require', show_default=True, expose_value=True, type=str,
            help='Specify an allocation type, include, exclude or require')
@click.pass_context
def allocation(ctx, rule, type):
    """Index Allocation"""
    if not rule:
        click.echo('{0}'.format(ctx.get_help()))
        click.echo(click.style('Missing required parameter --rule', fg='red', bold=True))
        sys.exit(1)

    if type not in ['require', 'include', 'exclude']:
        click.echo(click.style('--type can only be one of: require, include  exclude', fg='red', bold=True))
        sys.exit(1)

allocation.add_command(indices)



import click
import sys
import re
import time
import logging
import json
from .utils import *

import elasticsearch
from ..api import *

logger = logging.getLogger(__name__)

# Elasticsearch versions supported
version_max  = (2, 0, 0)
version_min = (1, 0, 0)

REGEX_MAP = {
    'timestring': r'^.*{0}.*$',
    'newer_than': r'(?P<date>{0})',
    'older_than': r'(?P<date>{0})',
    'prefix': r'^{0}.*$',
    'suffix': r'^.*{0}$',
}

def countdown(seconds):
    """Display an inline countdown to stdout."""
    for i in range(seconds,0,-1):
        sys.stdout.write(str(i) + ' ')
        sys.stdout.flush()
        time.sleep(1)

class LogstashFormatter(logging.Formatter):
    # The LogRecord attributes we want to carry over to the Logstash message,
    # mapped to the corresponding output key.
    WANTED_ATTRS = {'levelname': 'loglevel',
                    'funcName': 'function',
                    'lineno': 'linenum',
                    'message': 'message',
                    'name': 'name'}

    def converter(self, timevalue):
        return time.gmtime(timevalue)

    def format(self, record):
        timestamp = '%s.%03dZ' % (
            self.formatTime(record, datefmt='%Y-%m-%dT%H:%M:%S'), record.msecs)
        result = {'message': record.getMessage(),
                  '@timestamp': timestamp}
        for attribute in set(self.WANTED_ATTRS).intersection(record.__dict__):
            result[self.WANTED_ATTRS[attribute]] = getattr(record, attribute)
        return json.dumps(result, sort_keys=True)

class Whitelist(logging.Filter):
    def __init__(self, *whitelist):
        self.whitelist = [logging.Filter(name) for name in whitelist]

    def filter(self, record):
        return any(f.filter(record) for f in self.whitelist)

def exit_msg(success):
    """
    Display a message corresponding to whether the job completed successfully or
    not, then exit.
    """
    if success:
        logger.info("Job completed successfully.")
    else:
        logger.warn("Job did not complete successfully.")
    sys.exit(0) if success else sys.exit(1)

def show_dry_run(client, items, command, type=None):
    """
    Log dry run output with the command which would have been executed.
    """
    logger.info("DRY RUN MODE.  No changes will be made.")
    for item in items:
        if type == 'indices':
            logger.info("DRY RUN: {0}: {1}{2}".format(command, item, ' (CLOSED)' if index_closed(client, item) else ''))
        else:
            logger.info("DRY RUN: {0}: {1}".format(command, item))

def check_version(client):
    """
    Verify version is within acceptable range.  Exit with error if it is not.

    :arg client: The Elasticsearch client connection
    """
    version_number = get_version(client)
    logger.debug('Detected Elasticsearch version {0}'.format(".".join(map(str,version_number))))
    if version_number >= version_max or version_number < version_min:
        click.echo(click.style('Expected Elasticsearch version range > {0} < {1}'.format(".".join(map(str,version_min)),".".join(map(str,version_max))), fg='red'))
        click.echo(click.style('ERROR: Incompatible with version {0} of Elasticsearch.  Exiting.'.format(".".join(map(str,version_number))), fg='red', bold=True))
        sys.exit(1)

def check_master(client, master_only=False):
    """
    Check if master node.  If not, exit with error code
    """
    if master_only and not is_master_node(client):
        logger.info('Master-only flag detected. Connected to non-master node. Aborting.')
        sys.exit(9)

def get_client(**kwargs):
    """Return an Elasticsearch client using the provided parameters

    """
    kwargs['master_only'] = False if not 'master_only' in kwargs else kwargs['master_only']
    kwargs['use_ssl'] = False if not 'use_ssl' in kwargs else kwargs['use_ssl']
    kwargs['ssl_no_validate'] = False if not 'ssl_no_validate' in kwargs else kwargs['ssl_no_validate']
    kwargs['certificate'] = False if not 'certificate' in kwargs else kwargs['certificate']
    logger.debug("kwargs = {0}".format(kwargs))
    master_only = kwargs.pop('master_only')
    if kwargs['use_ssl']:
        if kwargs['ssl_no_validate']:
            kwargs['verify_certs'] = False # Not needed, but explicitly defined
        else:
            logger.info('Attempting to verify SSL certificate.')
            # If user provides a certificate:
            if kwargs['certificate']:
                kwargs['verify_certs'] = True
                kwargs['ca_certs'] = kwargs['certificate']
            else: # Try to use certifi certificates:
                try:
                    import certifi
                    kwargs['verify_certs'] = True
                    kwargs['ca_certs'] = certifi.where()
                except ImportError:
                    logger.warn('Unable to verify SSL certificate.')
    try:
        client = elasticsearch.Elasticsearch(**kwargs)
        # Verify the version is acceptable.
        check_version(client)
        # Verify "master_only" status, if applicable
        check_master(client, master_only=master_only)
        return client
    except Exception:
        click.echo(click.style('ERROR: Connection failure.', fg='red', bold=True))
        sys.exit(1)

def override_timeout(ctx):
    """
    Override the default timeout for optimize and snapshot operations if the
    default value of 30 is provided at the command-line.
    """
    timeout = 21600
    if ctx.parent.info_name in ['optimize', 'snapshot']:
        if ctx.parent.parent.params['timeout'] == 30:
            logger.warn('Overriding default connection timeout.  New timeout: {0}'.format(timeout))
            ctx.parent.parent.params['timeout'] = timeout

def filter_callback(ctx, param, value):
    """
    Append a dict to ctx.obj['filters'] based on the arguments
    """
    # Stop here if None or empty value, but zero values are okay
    if value == 0:
        argdict = {}
    elif not value:
        return value
    else:
        argdict = {}

    if param.name in ['older_than', 'newer_than']:
        if not ctx.params['time_unit'] :
            click.echo(click.style("Parameters --older-than and --newer-than require the --time-unit parameter", fg='red', bold=True))
            sys.exit(1)
        if not ctx.params['timestring']:
            click.echo(click.style("Parameters --older-than and --newer-than require the --timestring parameter", fg='red', bold=True))
            sys.exit(1)
        argdict = {  "groupname":'date', "time_unit":ctx.params["time_unit"],
                    "timestring": ctx.params['timestring'], "value": value,
                    "method": param.name }
        date_regex = get_date_regex(ctx.params['timestring'])
        regex = REGEX_MAP[param.name].format(date_regex)
    elif param.name == 'regex':
        regex = r'{0}'.format(value)
    elif param.name in ['prefix', 'suffix']:
        regex = REGEX_MAP[param.name].format(value)

    if param.name == 'exclude':
        for e in value:
            argdict = {}
            argdict['pattern'] = '{0}'.format(e)
            argdict['exclude'] = True
            ctx.obj['filters'].append(argdict)
            logger.debug("Added filter: {0}".format(argdict))
    else:
        logger.debug("REGEX = {0}".format(regex))
        argdict['pattern'] = regex
        ctx.obj['filters'].append(argdict)
        logger.debug("Added filter: {0}".format(argdict))
    logger.debug("New list of filters: {0}".format(ctx.obj['filters']))
    return value

def in_list(values, source_list):
    """
    Return a list of values found inside source_list.

    While a list comprehension is faster, it doesn't log failures.

    :arg values: A list of items to compare to the ``source_list``
    :arg source_list: A list of items
    """
    retval = []
    for v in values:
        if v in source_list:
            logger.info('Adding {0} from command-line argument'.format(v))
            retval.append(v)
        else:
            logger.warn('{0} not found!'.format(v))
    return retval

def do_command(client, command, indices, params=None, master_timeout=30000):
    """
    Do the command.
    """
    if command == "alias":
        return alias(
                client, indices, alias=params['name'], remove=params['remove']
               )
    if command == "allocation":
        return allocation(client, indices, rule=params['rule'], allocation_type=params['type'] )
    if command == "bloom":
        return bloom(client, indices, delay=params['delay'])
    if command == "close":
        return close(client, indices)
    if command == "delete":
        return delete(client, indices, master_timeout)
    if command == "open":
        return opener(client, indices)
    if command == "optimize":
        return optimize(
                client, indices, max_num_segments=params['max_num_segments'],
                delay=params['delay'], request_timeout=params['request_timeout']
               )
    if command == "replicas":
        return replicas(client, indices, replicas=params['count'])
    if command == "seal":
        return seal(client, indices)
    if command == "snapshot":
        return create_snapshot(
                client, indices=indices, name=params['name'],
                prefix=params['prefix'], repository=params['repository'],
                ignore_unavailable=params['ignore_unavailable'],
                include_global_state=params['include_global_state'],
                partial=params['partial'],
                wait_for_completion=params['wait_for_completion'],
                request_timeout=params['request_timeout'],
                skip_repo_validation=params['skip_repo_validation'],
               )



from datetime import datetime, timedelta
from unittest import TestCase
from mock import Mock
import sys
import click
from click import testing as clicktest

import logging
logger = logging.getLogger(__name__)

import curator

named_indices  = [ "index1", "index2" ]
named_alias    = 'alias_name'
alias_retval   = { "pre_aliased_index": { "aliases" : { named_alias : { }}}}
aliases_retval = {
    "index1": { "aliases" : { named_alias : { } } },
    "index2": { "aliases" : { named_alias : { } } },
    }
fake_fail      = Exception('Simulated Failure')
repo_name      = 'repo_name'
test_repo      = {repo_name: {'type': 'fs', 'settings': {'compress': 'true', 'location': '/tmp/repos/repo_name'}}}
test_repos     = {'TESTING': {'type': 'fs', 'settings': {'compress': 'true', 'location': '/tmp/repos/TESTING'}},
                  repo_name: {'type': 'fs', 'settings': {'compress': 'true', 'location': '/rmp/repos/repo_name'}}}
snap_name      = 'snap_name'
snapshot       = { 'snapshots': [
                    {
                        'duration_in_millis': 60000, 'start_time': '2015-01-01T00:00:00.000Z',
                        'shards': {'successful': 4, 'failed': 0, 'total': 4},
                        'end_time_in_millis': 0, 'state': 'SUCCESS',
                        'snapshot': snap_name, 'end_time': '2015-01-01T00:00:01.000Z',
                        'indices': named_indices,
                        'failures': [], 'start_time_in_millis': 0
                    }]}
snapshots       = { 'snapshots': [
                    {
                        'duration_in_millis': 60000, 'start_time': '2015-01-01T00:00:00.000Z',
                        'shards': {'successful': 4, 'failed': 0, 'total': 4},
                        'end_time_in_millis': 0, 'state': 'SUCCESS',
                        'snapshot': snap_name, 'end_time': '2015-01-01T00:00:01.000Z',
                        'indices': named_indices,
                        'failures': [], 'start_time_in_millis': 0
                    },
                    {
                        'duration_in_millis': 60000, 'start_time': '2015-01-01T00:00:02.000Z',
                        'shards': {'successful': 4, 'failed': 0, 'total': 4},
                        'end_time_in_millis': 0, 'state': 'SUCCESS',
                        'snapshot': 'snapshot2', 'end_time': '2015-01-01T00:00:03.000Z',
                        'indices': named_indices,
                        'failures': [], 'start_time_in_millis': 0
                    }]}
snap_body_all   = {
                    "ignore_unavailable": False,
                    "include_global_state": True,
                    "partial": False,
                    "indices" : "_all"
                  }
snap_body       = {
                    "ignore_unavailable": False,
                    "include_global_state": True,
                    "partial": False,
                    "indices" : "index1,index2"
                  }

class TestExitMsg(TestCase):
    def test_exit_msg_positive(self):
        with self.assertRaises(SystemExit) as cm:
            curator.exit_msg(True)
        self.assertEqual(cm.exception.code, 0)
    def test_exit_msg_negative(self):
        with self.assertRaises(SystemExit) as cm:
            curator.exit_msg(False)
        self.assertEqual(cm.exception.code, 1)

class TestCheckVersion(TestCase):
    def test_check_version_positive(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '1.1.1'} }
        self.assertIsNone(curator.check_version(client))
    def test_check_version_less_than(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '0.90.3'} }
        with self.assertRaises(SystemExit) as cm:
            curator.check_version(client)
        self.assertEqual(cm.exception.code, 1)
    def test_check_version_greater_than(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '2.0.1'} }
        with self.assertRaises(SystemExit) as cm:
            curator.check_version(client)
        self.assertEqual(cm.exception.code, 1)

class TestCheckMaster(TestCase):
    def test_check_master_positive(self):
        client = Mock()
        client.nodes.info.return_value = {
            'nodes': { "foo" : "bar"}
        }
        client.cluster.state.return_value = {
            "master_node" : "foo"
        }
        self.assertIsNone(curator.check_master(client, master_only=True))
    def test_check_master_negative(self):
        client = Mock()
        client.nodes.info.return_value = {
            'nodes': { "bad" : "mojo"}
        }
        client.cluster.state.return_value = {
            "master_node" : "foo"
        }
        with self.assertRaises(SystemExit) as cm:
            curator.check_master(client, master_only=True)
        self.assertEqual(cm.exception.code, 9)

class TestInList(TestCase):
    def test_in_list_positive(self):
        v = ['a', 'b']
        s = ['a', 'b', 'c', 'd']
        self.assertEqual(v, curator.in_list(v, s))
    def test_in_list_negative(self):
        v = ['a', 'b', 'q']
        s = ['a', 'b', 'c', 'd']
        self.assertEqual(['a', 'b'], curator.in_list(v, s))

class TestGetClient(TestCase):
    def test_certificate_logic(self):
        client = Mock()
        kwargs = { 'use_ssl' : True, 'certificate' : 'mycert.pem' }
        with self.assertRaises(SystemExit) as cm:
            curator.get_client(**kwargs)
            self.assertEqual(sys.stdout.getvalue(),'ERROR: Connection failure.\n')
        self.assertEqual(cm.exception.code, 1)



from .utils import *
import elasticsearch
import logging
logger = logging.getLogger(__name__)

def add_to_alias(client, index_name, alias=None):
    """
    Add indicated index to the specified alias.

    :arg client: The Elasticsearch client connection
    :arg index_name: The index name
    :arg alias: Alias name to operate on.
    :rtype: bool
    """
    if check_csv(index_name):
        logger.error("Must specify only a single index as an argument.")
        return False
    if not client.indices.exists(index_name):
        logger.error("Index {0} not found.".format(index_name))
        return False
    if not alias: # This prevents _all from being aliased by accident...
        logger.error('No alias provided.')
        return False
    if not client.indices.exists_alias(alias):
        indices_in_alias = []
        logger.info('Alias {0} not found.  Creating...'.format(alias))
    else:
        indices_in_alias = get_alias(client, alias)
    if not index_name in indices_in_alias:
        if index_closed(client, index_name):
            logger.error('Failed to add index {0} to alias {1} because it is closed.'.format(index_name, alias))
            return False
        else:
            try:
                client.indices.update_aliases(body={'actions': [{ 'add': { 'index': index_name, 'alias': alias}}]})
                return True
            except Exception as e:
                logger.error("Error adding index {0} to alias {1}.  Exception: {2}  Run with --debug flag and/or check Elasticsearch logs for more information.".format(index_name, alias, e))
                return False
    else:
        logger.info('Skipping index {0}: Index already exists in alias {1}...'.format(index_name, alias))
        return True

def remove_from_alias(client, index_name, alias=None):
    """
    Remove the indicated index from the specified alias.

    :arg client: The Elasticsearch client connection
    :arg index_name: The index name
    :arg alias: Alias name to operate on.
    :rtype: bool
    """
    if check_csv(index_name):
        logger.error("Must specify only a single index as an argument.")
        return False
    if not alias:
        logger.error('No alias provided.')
        return False
    indices_in_alias = get_alias(client, alias)
    if not indices_in_alias:
        logger.error("Index {0} not found in alias {1}".format(index_name, alias))
        return False
    if index_name in indices_in_alias:
        try:
            client.indices.update_aliases(body={'actions': [{ 'remove': { 'index': index_name, 'alias': alias}}]})
            return True
        except Exception as e:
            logger.error("Error removing index {0} from alias {1}.  Exception: {2}  Run with --debug flag and/or check Elasticsearch logs for more information.".format(index_name, alias, e))
            return False
    else:
        logger.warn('Index {0} does not exist in alias {1}; skipping.'.format(index_name, alias))
        return False

def alias(client, indices, alias=None, remove=False):
    """
    Helper method called by the CLI.

    :arg client: The Elasticsearch client connection
    :arg indices: A list of indices to act on
    :arg alias: Alias name to operate on.
    :arg remove: If true, remove the alias.
    :rtype: bool
    """
    retval = True
    for i in ensure_list(indices):
        if remove:
            success = remove_from_alias(client, i, alias=alias)
        else:
            success = add_to_alias(client, i, alias=alias)
        # if we fail once, we fail completely
        if not success:
            retval = False
    return retval



from datetime import datetime, timedelta
from unittest import TestCase
from mock import patch, Mock
import click
import sys
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

import elasticsearch
import curator

fake_fail      = Exception('Simulated Failure')
repo_name      = 'repo_name'

class TestCreateRepoBody(TestCase):
    def test_create_repo_body_missing_repo_type(self):
        with self.assertRaises(SystemExit) as cm:
            curator.create_repo_body()
        self.assertEqual(cm.exception.code, 1)

    def test_create_repo_body_s3(self):
        body = curator.create_repo_body(repo_type='s3')
        self.assertEqual(body['type'], 's3')

class TestCreateRepository(TestCase):
    def test_create_repository_missing_arg(self):
        client = Mock()
        with self.assertRaises(SystemExit) as cm:
            curator.create_repository(client)
        self.assertEqual(cm.exception.code, 1)

    def test_create_repository_empty_result_call(self):
        client = Mock()
        client.snapshot.get_repository.return_value = None
        self.assertTrue(curator.create_repository(client, repository="repo", repo_type="fs"))

    def test_create_repository_repo_not_in_results(self):
        client = Mock()
        client.snapshot.get_repository.return_value = {'not_your_repo':{'foo':'bar'}}
        self.assertTrue(curator.create_repository(client, repository="repo", repo_type="fs"))

    def test_create_repository_repo_already_in_results(self):
        client = Mock()
        client.snapshot.get_repository.return_value = {'repo':{'foo':'bar'}}
        with self.assertRaises(SystemExit) as cm:
            curator.create_repository(client, repository="repo", repo_type="fs")
        self.assertEqual(cm.exception.code, 1)

    def test_create_repository_exception(self):
        client = Mock()
        client.snapshot.get_repository.return_value = {'not_your_repo':{'foo':'bar'}}
        client.snapshot.create_repository.side_effect = elasticsearch.TransportError(500, "Error message", {"message":"Error"})
        self.assertRaises(Exception, curator.create_repository(client, repository="repo", repo_type="fs"))

class TestVerifyRepository(TestCase):
    def test_verify_repository_missing_arg(self):
        client = Mock()
        with self.assertRaises(SystemExit) as cm:
            curator.verify_repository(client)
        self.assertEqual(cm.exception.code, 1)

    def test_verify_repository_in_results(self):
        client = Mock()
        client.snapshot.get_repository.return_value = {'repo':{'foo':'bar'}}
        self.assertTrue(curator.verify_repository(client, repository="repo"))

    def test_verify_repository_repo_not_in_results(self):
        client = Mock()
        client.snapshot.get_repository.return_value = {'not_your_repo':{'foo':'bar'}}
        self.assertFalse(curator.verify_repository(client, repository="repo"))

class TestDeleteCallback(TestCase):
    def test_delete_callback_no_value(self):
        ctx = Mock()
        param = None
        value = None
        self.assertEqual(curator.delete_callback(ctx, param, value), None)

    def test_delete_callback_with_value(self):
        ctx = Mock()
        param = None
        value = True
        ctx.abort.return_value = None
        self.assertEqual(curator.delete_callback(ctx, param, value), None)



from datetime import datetime, timedelta
from unittest import TestCase
from mock import Mock

from curator import api as curator

named_index    = 'index_name'
named_indices  = [ "index1", "index2" ]
open_index     = {'metadata': {'indices' : { named_index : {'state' : 'open'}}}}
closed_index   = {'metadata': {'indices' : { named_index : {'state' : 'close'}}}}
open_indices   = { 'metadata': { 'indices' : { 'index1' : { 'state' : 'open' },
                                               'index2' : { 'state' : 'open' }}}}
closed_indices = { 'metadata': { 'indices' : { 'index1' : { 'state' : 'close' },
                                               'index2' : { 'state' : 'close' }}}}
fake_fail      = Exception('Simulated Failure')
indices_space  = { 'indices' : {
        'index1' : { 'index' : { 'primary_size_in_bytes': 1083741824 }},
        'index2' : { 'index' : { 'primary_size_in_bytes': 1083741824 }}}}
re_test_indices = [
    "logstash-2014.12.31", "logstash-2014.12.30", "logstash-2014.12.29",
    ".marvel-2015.12.31", ".marvel-2015.12.30", ".marvel-2015.12.29",
    "2014.11.30", "2014.11.29", "2014.12.28",
    "2014.10.31-buh", "2014.10.30-buh", "2014.10.29-buh",
    "2015-01-01", "2015-01-02", "2015-01-03",
    "20150101", "20150102", "20150103",
    "foo", "bar", "baz", "neeble",
    "logstash-foo", "logstash-bar",
    "foo-logstash", "bar-logstash",
    "foologstash", "barlogstash",
    "logstashfoo", "logstashbar",
    ]

sized           = { u'index1': {u'settings': {u'index': {u'number_of_replicas': u'0'} } },
                    u'index2': {u'settings': {u'index': {u'number_of_replicas': u'0'} } } }

class FilterBySpace(TestCase):
    def test_filter_by_space_param_check(self):
        client = Mock()
        # Testing for the omission of the disk_space param
        self.assertFalse(curator.filter_by_space(client, named_indices))
    def test_filter_by_space_all_indices_closed(self):
        client = Mock()
        ds = 100.0
        client.cluster.state.return_value = closed_indices
        self.assertEqual([], curator.filter_by_space(client, named_indices, disk_space=ds))
    def test_filter_by_space_no_deletions_positive(self):
        client = Mock()
        ds = 10.0
        client.cluster.state.return_value = open_indices
        client.indices.get_settings.return_value = sized
        # Build return value of over 1G in size for each index
        client.indices.status.return_value = indices_space
        self.assertEqual([], curator.filter_by_space(client, named_indices, disk_space=ds))
    def test_filter_by_space_one_deletion(self):
        client = Mock()
        ds = 2.0
        client.cluster.state.return_value = open_indices
        client.indices.get_settings.return_value = sized
        # Build return value of over 1G in size for each index
        client.indices.status.return_value = indices_space
        self.assertEqual(["index1"], curator.filter_by_space(client, named_indices, disk_space=ds))
    def test_filter_by_space_one_deletion_no_reverse(self):
        client = Mock()
        ds = 2.0
        client.cluster.state.return_value = open_indices
        client.indices.get_settings.return_value = sized
        # Build return value of over 1G in size for each index
        client.indices.status.return_value = indices_space
        self.assertEqual(["index2"], curator.filter_by_space(client, named_indices, disk_space=ds, reverse=False))

class TestBuildFilter(TestCase):
    def test_build_filter_missing_param_kindOf(self):
        self.assertEqual({}, curator.build_filter())
    def test_build_filter_missing_param_value(self):
        self.assertEqual({}, curator.build_filter(kindOf='regex'))
    def test_build_filter_missing_param_time_unit(self):
        self.assertEqual({}, curator.build_filter(kindOf='older_than', value=5))
    def test_build_filter_missing_param_timestring(self):
        self.assertEqual(
            {},
            curator.build_filter(
                kindOf='older_than', value=5, time_unit='days'
            ),
        )
    def test_build_filter_newer_than_positive(self):
        self.assertEqual(
            {
                'pattern': '(?P<date>\\d{4}\\.\\d{2}\\.\\d{2})', 'value': 5,
                'groupname': 'date', 'time_unit': 'days',
                'timestring': '%Y.%d.%m', 'method': 'newer_than'
            },
            curator.build_filter(
                kindOf='newer_than', value=5, time_unit='days',
                timestring='%Y.%d.%m',
            ),
        )
    def test_build_filter_newer_than_zero(self):
        self.assertEqual(
            {
                'pattern': '(?P<date>\\d{4}\\.\\d{2}\\.\\d{2})', 'value': 0,
                'groupname': 'date', 'time_unit': 'days',
                'timestring': '%Y.%d.%m', 'method': 'newer_than'
            },
            curator.build_filter(
                kindOf='newer_than', value=0, time_unit='days',
                timestring='%Y.%d.%m',
            ),
        )
    def test_build_filter_regex_positive(self):
        self.assertEqual(
            {'pattern': '^logs'},
            curator.build_filter(kindOf='regex', value='^logs'),
        )
    def test_build_filter_exclude_positive(self):
        self.assertEqual(
            {'pattern': '^logs', 'exclude': True},
            curator.build_filter(kindOf='exclude', value='^logs'),
        )
    def test_build_filter_prefix_positive(self):
        self.assertEqual(
            {'pattern': '^logs.*$'},
            curator.build_filter(kindOf='prefix', value='logs'),
        )
    def test_build_filter_suffix_positive(self):
        self.assertEqual(
            {'pattern': '^.*logs$'},
            curator.build_filter(kindOf='suffix', value='logs'),
        )
    def test_build_filter_timestring_positive(self):
        self.assertEqual(
            {'pattern': '^.*\\d{4}\\.\\d{2}\\.\\d{2}.*$'},
            curator.build_filter(kindOf='timestring', value='%Y.%m.%d'),
        )

class TestRegexIterate(TestCase):
    def test_apply_filter_missing_param_pattern(self):
        self.assertFalse(curator.apply_filter(re_test_indices))
    # The apply_filter method filters a list of indices based on regex patterns
    def test_apply_filter_filter_none(self):
        pattern = r'.*'
        self.assertEqual(
            re_test_indices, curator.apply_filter(re_test_indices,
            pattern=pattern)
        )
    def test_apply_filter_prefix(self):
        pattern = r'^foo.*$'
        expected = [ 'foo', 'foo-logstash', 'foologstash' ]
        self.assertEqual(
            expected, curator.apply_filter(re_test_indices, pattern=pattern)
        )
    def test_apply_filter_suffix(self):
        pattern = r'^.*foo$'
        expected = [ 'foo', 'logstash-foo', 'logstashfoo' ]
        self.assertEqual(
            expected, curator.apply_filter(re_test_indices, pattern=pattern)
        )
    def test_apply_filter_timestring(self):
        pattern = r'^.*\d{4}.\d{2}.\d{2}.*$'
        expected = [
            'logstash-2014.12.31', 'logstash-2014.12.30', 'logstash-2014.12.29',
            '.marvel-2015.12.31', '.marvel-2015.12.30', '.marvel-2015.12.29',
            '2014.11.30', '2014.11.29', '2014.12.28', '2014.10.31-buh',
            '2014.10.30-buh', '2014.10.29-buh', '2015-01-01', '2015-01-02',
            '2015-01-03',
            ]
        self.assertEqual(
            expected, curator.apply_filter(re_test_indices,
            pattern=pattern)
        )
    def test_apply_filter_newer_than(self):
        pattern = r'(?P<date>\d{4}.\d{2}.\d{2})'
        expected = [
            'logstash-2014.12.31', 'logstash-2014.12.30', 'logstash-2014.12.29',
            '.marvel-2015.12.31', '.marvel-2015.12.30', '.marvel-2015.12.29',
            '2014.12.28',
            ]
        t = datetime(2014, 12, 1, 2, 34, 56)
        self.assertEqual(expected,
            curator.apply_filter(re_test_indices, pattern=pattern,
                groupname='date', timestring='%Y.%m.%d', time_unit='days',
                method='newer_than', value=2, utc_now=t
            )
        )
    def test_apply_filter_newer_than_negative_value(self):
        pattern = r'(?P<date>\d{4}.\d{2}.\d{2})'
        expected = [
            '.marvel-2015.12.31', '.marvel-2015.12.30', '.marvel-2015.12.29'
            ]
        t = datetime(2014, 12, 1, 2, 34, 56)
        self.assertEqual(expected,
            curator.apply_filter(
                re_test_indices, pattern=pattern, groupname='date',
                timestring='%Y.%m.%d', time_unit='days', method='newer_than',
                value=-30, utc_now=t
            )
        )
    def test_apply_filter_older_than_date_only(self):
        pattern = r'(?P<date>\d{4}-\d{2}-\d{2})'
        expected = [
            '2015-01-01', '2015-01-02', '2015-01-03',
            ]
        t = datetime(2015, 2, 1, 2, 34, 56)
        self.assertEqual(expected,
            curator.apply_filter(
                re_test_indices, pattern=pattern, groupname='date',
                timestring='%Y-%m-%d', time_unit='days', method='older_than',
                value=2, utc_now=t
            )
        )
    def test_apply_filter_older_than_with_prefix_and_suffix(self):
        pattern = r'(?P<date>\d{4}.\d{2}.\d{2})'
        expected = [
            'logstash-2014.12.31', 'logstash-2014.12.30', 'logstash-2014.12.29',
            '2014.11.30', '2014.11.29', '2014.12.28', '2014.10.31-buh',
            '2014.10.30-buh', '2014.10.29-buh'
            ]
        t = datetime(2015, 2, 1, 2, 34, 56)
        self.assertEqual(expected,
            curator.apply_filter(
                re_test_indices, pattern=pattern, groupname='date',
                timestring='%Y.%m.%d', time_unit='days', method='older_than',
                value=2, utc_now=t
            )
        )
    def test_apply_filter_exclude(self):
        pattern = r'201|logstash'
        expected = ['foo', 'bar', 'baz', 'neeble']
        self.assertEqual(
            expected, curator.apply_filter(re_test_indices,
            pattern=pattern, exclude=True)
        )

class TestGetDateRegex(TestCase):
    def test_get_date_regex_arbitrary(self):
        self.assertEqual('\\a\\a\\a\\a', curator.get_date_regex('aaaa'))
    def test_get_date_regex_arbitrary_with_percent(self):
        self.assertEqual('\\a\\a\\a\\a', curator.get_date_regex('%a%a%a%a'))
    def test_get_date_regex_date_map(self):
        self.assertEqual('\\d{4}\\.\\d{2}\\.\\d{2}\\-\\d{2}\\-\\d{2}\\d{2}\\d{2}\\d{2}\\d{2}',
            curator.get_date_regex('%Y.%y.%m-%W-%U%d%H%M%S'))
    def test_get_date_regex_date_map2(self):
        self.assertEqual('\\d{4}\\-\\d{3}',
            curator.get_date_regex('%Y-%j'))


class TestGetIndexTime(TestCase):
    def test_get_datetime_week_fix_W(self):
        utc_now  = datetime(2015, 2, 1, 2, 34, 56)
        expected = datetime(2015, 1, 26, 0, 00, 00)
        weeknow  = utc_now.strftime('%Y-%W')
        self.assertEqual(expected, curator.get_datetime(weeknow, '%Y-%W'))
    def test_get_datetime_week_fix_U(self):
        utc_now  = datetime(2015, 2, 21, 2, 34, 56)
        expected = datetime(2015, 2, 16, 0, 00, 00)
        weeknow  = utc_now.strftime('%Y-%U')
        self.assertEqual(expected, curator.get_datetime(weeknow, '%Y-%U'))
    def test_get_datetime_month_fix_positive(self):
        utc_now  = datetime(2015, 2, 22, 2, 34, 56)
        expected = datetime(2015, 2, 1, 0, 00, 00)
        weeknow  = utc_now.strftime('%Y-%m')
        self.assertEqual(expected, curator.get_datetime(weeknow, '%Y-%m'))
    def test_get_datetime_month_fix_negative(self):
        utc_now  = datetime(2015, 2, 22, 2, 34, 56)
        expected = datetime(2015, 2, 22, 0, 00, 00)
        weeknow  = utc_now.strftime('%Y-%m-%d')
        self.assertEqual(expected, curator.get_datetime(weeknow, '%Y-%m-%d'))

class TestGetTargetMonth(TestCase):
    def test_get_target_month_same_year(self):
        before = datetime(2015, 2, 1, 2, 34, 56)
        after  = datetime(2015, 1, 1, 0, 0, 0)
        self.assertEqual(after, curator.get_target_month(1, utc_now=before))
    def test_get_target_negative_value_month_same_year(self):
        before = datetime(2015, 2, 1, 2, 34, 56)
        after  = datetime(2015, 3, 1, 0, 0, 0)
        self.assertEqual(after, curator.get_target_month(-1, utc_now=before))
    def test_get_target_month_previous_year(self):
        before = datetime(2015, 2, 1, 2, 34, 56)
        after  = datetime(2014, 12, 1, 0, 0, 0)
        self.assertEqual(after, curator.get_target_month(2, utc_now=before))
    def test_get_target_negative_value_month_next_year(self):
        before = datetime(2015, 2, 1, 2, 34, 56)
        after  = datetime(2016, 1, 1, 0, 0, 0)
        self.assertEqual(after, curator.get_target_month(-11, utc_now=before))
    def test_month_bump_exception(self):
        datestamp = datetime(2015, 2, 1, 2, 34, 56)
        self.assertRaises(ValueError, curator.month_bump,datestamp, sign='foo')


class TestGetCutoff(TestCase):
    def test_get_cutoff_param_check(self):
        # Testing for the omission of the unit_count param
        self.assertFalse(curator.get_cutoff())
    def test_get_cutoff_non_integer(self):
        self.assertFalse(curator.get_cutoff(unit_count='foo'))
    def test_get_cutoff_weeks(self):
        fakenow = datetime(2015, 2, 3, 4, 5, 6)
        cutoff  = datetime(2015, 2, 2, 0, 0, 0)
        self.assertEqual(cutoff, curator.get_cutoff(1, time_unit='weeks', utc_now=fakenow))
    def test_get_cutoff_months(self):
        fakenow = datetime(2015, 2, 3, 4, 5, 6)
        cutoff  = datetime(2015, 1, 1, 0, 0, 0)
        self.assertEqual(cutoff, curator.get_cutoff(1, time_unit='months', utc_now=fakenow))
    def test_get_cutoff_negtive_value_weeks(self):
        fakenow = datetime(2015, 2, 3, 4, 5, 6)
        cutoff  = datetime(2015, 3, 9, 0, 0, 0)
        self.assertEqual(cutoff, curator.get_cutoff(-5, time_unit='weeks', utc_now=fakenow))
    def test_get_cutoff_negtive_value_months(self):
        fakenow = datetime(2015, 2, 3, 4, 5, 6)
        cutoff  = datetime(2015, 7, 1, 0, 0, 0)
        self.assertEqual(cutoff, curator.get_cutoff(-5, time_unit='months', utc_now=fakenow))

class TestTimestampCheck(TestCase):
    # Only the final else statement was not listed as "covered" by nose
    def test_timestamp_check_else(self):
        ts = '%Y.%m.%d'
        tu = 'days'
        v  = 30
        timestamp = '2015.01.01'
        utc_now = datetime(2015, 1, 5)
        self.assertFalse(curator.timestamp_check(timestamp, timestring=ts,
            time_unit=tu, value=v, utc_now=utc_now))
    def test_timestamp_check_get_cutoff_with_none(self):
        ts = '%Y.%m.%d'
        tu = 'days'
        v  = None
        timestamp = '2015.01.01'
        utc_now = datetime(2015, 1, 5)
        self.assertFalse(curator.timestamp_check(timestamp, timestring=ts,
            time_unit=tu, value=v, utc_now=utc_now))
    def test_timestamp_check_get_cutoff_with_non_int(self):
        ts = '%Y.%m.%d'
        tu = 'days'
        v  = 'foo'
        timestamp = '2015.01.01'
        utc_now = datetime(2015, 1, 5)
        self.assertFalse(curator.timestamp_check(timestamp, timestring=ts,
            time_unit=tu, value=v, utc_now=utc_now))



from datetime import datetime, timedelta
from unittest import TestCase
from mock import Mock
import elasticsearch

from curator import api as curator

named_index    = 'index_name'
closed_index   = {'metadata': {'indices' : { named_index : {'state' : 'close'}}}}
named_indices  = [ "index1", "index2" ]
named_alias    = 'alias_name'
alias_retval   = { "pre_aliased_index": { "aliases" : { named_alias : { }}}}
aliases_retval = {
    "index1": { "aliases" : { named_alias : { } } },
    "index2": { "aliases" : { named_alias : { } } },
    }
fake_fail      = Exception('Simulated Failure')
repo_name      = 'repo_name'
test_repo      = {repo_name: {'type': 'fs', 'settings': {'compress': 'true', 'location': '/tmp/repos/repo_name'}}}
test_repos     = {'TESTING': {'type': 'fs', 'settings': {'compress': 'true', 'location': '/tmp/repos/TESTING'}},
                  repo_name: {'type': 'fs', 'settings': {'compress': 'true', 'location': '/rmp/repos/repo_name'}}}
snap_name      = 'snap_name'
snapshot       = { 'snapshots': [
                    {
                        'duration_in_millis': 60000, 'start_time': '2015-01-01T00:00:00.000Z',
                        'shards': {'successful': 4, 'failed': 0, 'total': 4},
                        'end_time_in_millis': 0, 'state': 'SUCCESS',
                        'snapshot': snap_name, 'end_time': '2015-01-01T00:00:01.000Z',
                        'indices': named_indices,
                        'failures': [], 'start_time_in_millis': 0
                    }]}
snapshots       = { 'snapshots': [
                    {
                        'duration_in_millis': 60000, 'start_time': '2015-01-01T00:00:00.000Z',
                        'shards': {'successful': 4, 'failed': 0, 'total': 4},
                        'end_time_in_millis': 0, 'state': 'SUCCESS',
                        'snapshot': snap_name, 'end_time': '2015-01-01T00:00:01.000Z',
                        'indices': named_indices,
                        'failures': [], 'start_time_in_millis': 0
                    },
                    {
                        'duration_in_millis': 60000, 'start_time': '2015-01-01T00:00:02.000Z',
                        'shards': {'successful': 4, 'failed': 0, 'total': 4},
                        'end_time_in_millis': 0, 'state': 'SUCCESS',
                        'snapshot': 'snapshot2', 'end_time': '2015-01-01T00:00:03.000Z',
                        'indices': named_indices,
                        'failures': [], 'start_time_in_millis': 0
                    }]}
snap_body_all   = {
                    "ignore_unavailable": False,
                    "include_global_state": True,
                    "partial": False,
                    "indices" : "_all"
                  }
snap_body       = {
                    "ignore_unavailable": False,
                    "include_global_state": True,
                    "partial": False,
                    "indices" : "index1,index2"
                  }

class TestGetAlias(TestCase):
    def test_get_alias_positive(self):
        client = Mock()
        client.indices.exists_alias.return_value = True
        client.indices.get_alias.return_value = aliases_retval
        retval = sorted(curator.get_alias(client, named_alias))
        self.assertEqual(named_indices, retval)
    def test_get_alias_negative(self):
        client = Mock()
        client.indices.exists_alias.return_value = False
        self.assertFalse(curator.get_alias(client, named_alias))

class TestEnsureList(TestCase):
    def test_ensure_list_returns_lists(self):
        l = ["a", "b", "c", "d"]
        e = ["a", "b", "c", "d"]
        self.assertEqual(e, curator.ensure_list(l))
        l = "abcd"
        e = ["abcd"]
        self.assertEqual(e, curator.ensure_list(l))
        l = [["abcd","defg"], 1, 2, 3]
        e = [["abcd","defg"], 1, 2, 3]
        self.assertEqual(e, curator.ensure_list(l))
        l = {"a":"b", "c":"d"}
        e = [{"a":"b", "c":"d"}]
        self.assertEqual(e, curator.ensure_list(l))

class TestTo_CSV(TestCase):
    def test_to_csv_will_return_csv(self):
        l = ["a", "b", "c", "d"]
        c = "a,b,c,d"
        self.assertEqual(c, curator.to_csv(l))
    def test_to_csv_will_return_single(self):
        l = ["a"]
        c = "a"
        self.assertEqual(c, curator.to_csv(l))
    def test_to_csv_will_return_None(self):
        l = []
        self.assertIsNone(curator.to_csv(l))

class TestCheckCSV(TestCase):
    def test_check_csv_positive(self):
        c = "1,2,3"
        self.assertTrue(curator.check_csv(c))
    def test_check_csv_negative(self):
        c = "12345"
        self.assertFalse(curator.check_csv(c))
    def test_check_csv_list(self):
        l = ["1", "2", "3"]
        self.assertTrue(curator.check_csv(l))
    def test_check_csv_unicode(self):
        u = u'test'
        self.assertFalse(curator.check_csv(u))
    def test_check_csv_wrong_value(self):
        v = 123
        with self.assertRaises(SystemExit) as cm:
            curator.check_csv(v)
        self.assertEqual(cm.exception.code, 1)

class TestPruneKibana(TestCase):
    def test_prune_kibana_positive(self):
        l = [
            "logstash-2015.02.25", "logstash-2015.02.24", ".kibana",
            ".marvel-kibana", "kibana-int", ".marvel-2015.02.25",
            ".marvel-2015.02.24",
            ]
        r = [
            "logstash-2015.02.25", "logstash-2015.02.24", ".marvel-2015.02.25",
            ".marvel-2015.02.24",
            ]
        self.assertEqual(r, curator.prune_kibana(l))
    def test_prune_kibana_negative(self):
        l = [
            "logstash-2015.02.25", "logstash-2015.02.24", ".marvel-2015.02.25",
            ".marvel-2015.02.24",
            ]
        r = [
            "logstash-2015.02.25", "logstash-2015.02.24", ".marvel-2015.02.25",
            ".marvel-2015.02.24",
            ]
        self.assertEqual(r, curator.prune_kibana(l))
    def test_prune_kibana_empty(self):
        l = [ ".kibana", ".marvel-kibana", "kibana-int", ]
        r = []
        self.assertEqual(r, curator.prune_kibana(l))

class TestGetVersion(TestCase):
    def test_positive(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '9.9.9'} }
        version = curator.get_version(client)
        self.assertEqual(version, (9,9,9))
    def test_negative(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '9.9.9'} }
        version = curator.get_version(client)
        self.assertNotEqual(version, (8,8,8))
    def test_dev_version_4_dots(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '9.9.9.dev'} }
        version = curator.get_version(client)
        self.assertEqual(version, (9,9,9))
    def test_dev_version_with_dash(self):
        client = Mock()
        client.info.return_value = {'version': {'number': '9.9.9-dev'} }
        version = curator.get_version(client)
        self.assertEqual(version, (9,9,9))

class TestOptimized(TestCase):
    def test_optimized_index_bad_csv(self):
        client = Mock()
        self.assertRaises(ValueError, curator.optimized, client, "a,b,c,d", max_num_segments=2)
    def test_optimized_index_missing_arg(self):
        client = Mock()
        self.assertRaises(ValueError, curator.optimized, client, named_index)
    def test_optimized_index_closed(self):
        client = Mock()
        client.cluster.state.return_value = closed_index
        self.assertTrue(curator.optimized(client, named_index, max_num_segments=2))

class TestIsMasterNode(TestCase):
    def test_positive(self):
        client = Mock()
        client.nodes.info.return_value = {
            'nodes': { "foo" : "bar"}
        }
        client.cluster.state.return_value = {
            "master_node" : "foo"
        }
        self.assertTrue(curator.is_master_node(client))
    def test_negative(self):
        client = Mock()
        client.nodes.info.return_value = {
            'nodes': { "bad" : "mojo"}
        }
        client.cluster.state.return_value = {
            "master_node" : "foo"
        }
        self.assertFalse(curator.is_master_node(client))

class TestGetIndexTime(TestCase):
    def test_get_datetime(self):
        for text, datestring, dt in [
            ('2014.01.19', '%Y.%m.%d', datetime(2014, 1, 19)),
            ('14.01.19', '%y.%m.%d', datetime(2014, 1, 19)),
            ('2014-01-19', '%Y-%m-%d', datetime(2014, 1, 19)),
            ('2010-12-29', '%Y-%m-%d', datetime(2010, 12, 29)),
            ('2012-12', '%Y-%m', datetime(2012, 12, 1)),
            ('2011.01', '%Y.%m', datetime(2011, 1, 1)),
            ('2014-28', '%Y-%W', datetime(2014, 7, 14)),
            ('2010.12.29.12', '%Y.%m.%d.%H', datetime(2010, 12, 29, 12)),
            ('2009101112136', '%Y%m%d%H%M%S', datetime(2009, 10, 11, 12, 13, 6)),
                ]:
            self.assertEqual(dt, curator.get_datetime(text, datestring))

class TestGetRepository(TestCase):
    def test_get_repository_missing_arg(self):
        client = Mock()
        client.snapshot.get_repository.return_value = {}
        self.assertEqual({}, curator.get_repository(client))
    def test_get_repository_positive(self):
        client = Mock()
        client.snapshot.get_repository.return_value = test_repo
        self.assertEqual(test_repo, curator.get_repository(client, repository=repo_name))
    def test_get_repository_transporterror_negative(self):
        client = Mock()
        client.snapshot.get_repository.side_effect = elasticsearch.TransportError
        self.assertFalse(curator.get_repository(client, repository=repo_name))
    def test_get_repository_notfounderror_negative(self):
        client = Mock()
        client.snapshot.get_repository.side_effect = elasticsearch.NotFoundError
        self.assertFalse(curator.get_repository(client, repository=repo_name))
    def test_get_repository__all_positive(self):
        client = Mock()
        client.snapshot.get_repository.return_value = test_repos
        self.assertEqual(test_repos, curator.get_repository(client))

class TestGetSnapshot(TestCase):
    def test_get_snapshot_missing_repository_arg(self):
        client = Mock()
        self.assertFalse(curator.get_snapshot(client, snapshot=snap_name))
    def test_get_snapshot_positive(self):
        client = Mock()
        client.snapshot.get.return_value = snapshot
        self.assertEqual(snapshot, curator.get_snapshot(client, repository=repo_name, snapshot=snap_name))
    def test_get_snapshot_transporterror_negative(self):
        client = Mock()
        client.snapshot.get_repository.return_value = test_repo
        client.snapshot.get.side_effect = elasticsearch.TransportError
        self.assertFalse(curator.get_snapshot(client, repository=repo_name, snapshot=snap_name))
    def test_get_snapshot_notfounderror_negative(self):
        client = Mock()
        client.snapshot.get_repository.return_value = test_repo
        client.snapshot.get.side_effect = elasticsearch.NotFoundError
        self.assertFalse(curator.get_snapshot(client, repository=repo_name, snapshot=snap_name))

class TestGetSnapshots(TestCase):
    def test_get_snapshots_missing_repository_arg(self):
        client = Mock()
        self.assertFalse(curator.get_snapshots(client))
    def test_get_snapshots_positive(self):
        client = Mock()
        client.snapshot.get.return_value = snapshot
        self.assertEqual(['snap_name'], curator.get_snapshots(client, repository=repo_name))
    def test_get_snapshots_multiple_positive(self):
        client = Mock()
        client.snapshot.get.return_value = snapshots
        self.assertEqual(['snap_name', 'snapshot2'], curator.get_snapshots(client, repository=repo_name))
    def test_get_snapshots_transporterror_negative(self):
        client = Mock()
        client.snapshot.get_repository.return_value = test_repo
        client.snapshot.get.side_effect = elasticsearch.TransportError
        self.assertFalse(curator.get_snapshots(client, repository=repo_name))
    def test_get_snapshots_notfounderror_negative(self):
        client = Mock()
        client.snapshot.get_repository.return_value = test_repo
        client.snapshot.get.side_effect = elasticsearch.NotFoundError
        self.assertFalse(curator.get_snapshots(client, repository=repo_name))

class TestCreateSnapshotBody(TestCase):
    def test_create_snapshot_body_empty_arg(self):
        self.assertFalse(curator.create_snapshot_body([]))
    def test_create_snapshot_body__all_positive(self):
        self.assertEqual(snap_body_all, curator.create_snapshot_body('_all'))
    def test_create_snapshot_body_positive(self):
        self.assertEqual(snap_body, curator.create_snapshot_body(named_indices))



from .utils import *
from .filter import *
import elasticsearch
import logging
logger = logging.getLogger(__name__)

def delete_indices(client, indices, master_timeout=30000):
    """
    Delete the indicated indices, including closed indices.

    :arg client: The Elasticsearch client connection
    :arg indices: A list of indices to act on
    :arg master_timeout: Number of milliseconds to wait for master node response
    :rtype bool:
    """
    indices = ensure_list(indices)
    try:
        logger.info("Deleting indices as a batch operation:")
        for i in indices:
            logger.info("---deleting index {0}".format(i))
        client.indices.delete(index=to_csv(indices), master_timeout=master_timeout)
        return True
    except Exception as e:
        logger.error("Error deleting one or more indices.  Run with --debug flag and/or check Elasticsearch logs for more information.")
        try:
            logger.error('Got a {0} response from Elasticsearch.  Error message: {1}'.format(e.status_code, e.error))
        except AttributeError:
            logger.error('Error message: {0}'.format(e))
        return False

def delete(client, indices, master_timeout=30000):
    """
    Helper method called by the CLI.  Tries up to 3x to delete indices.

    :arg client: The Elasticsearch client connection
    :arg indices: A list of indices to act on
    :arg master_timeout: Number of milliseconds to wait for master node response
    :rtype bool:
    """
    for count in range(1, 4): # Try 3 times
        logger.debug("master_timeout value: {0}".format(master_timeout))
        delete_indices(client, indices, master_timeout)
        result = [ i for i in indices if i in get_indices(client) ]
        if len(result) > 0:
            logger.error("Indices failed to delete:")
            for idx in result:
                logger.error("---{0}".format(idx))
        else:
            # break
            return True # We leave the loop here, if everything deleted.
        indices = result
    logger.error("Unable to delete indices after 3 attempts: {0}".format(result))
    return False # If we make it here, indices didn't delete after 3 tries.



from .utils import *
from .cli import *
from .alias import *
from .allocation import *
from .bloom import *
from .close import *
from .delete import *
from .opener import *
from .optimize import *
from .replicas import *
from .seal import *
from .show import *
from .snapshot import *
from .index_selection import *
from .snapshot_selection import *
from .es_repo_mgr import *



from .utils import *

def show(client, object_list, type=None):
    """
    Helper method called by the CLI.

    :arg client: The Elasticsearch client connection
    :arg object_list: A list of indices or snapshots to show
    :arg object_type: `indices` or `snapshots`
    :rtype: bool
    """
    for obj in ensure_list(object_list):
        if type == 'indices':
            print('{0}{1}'.format(obj, ' (CLOSED)' if index_closed(client, obj) else ''))
        else:
            print('{0}'.format(obj))
    return True



from .utils import *
from .seal import *
import elasticsearch
import logging
logger = logging.getLogger(__name__)

def close_indices(client, indices):
    """
    Close the indicated indices.  Perform a flush (a synced flush, if your
    Elasticsearch version supports it) before closing.
    This method will ignore unavailable (including closed) indices.

    :arg client: The Elasticsearch client connection
    :arg indices: A list of indices to act on
    :rtype: bool
    """
    indices = ensure_list(indices)
    if get_version(client) >= (1, 6, 0):
        seal_indices(client, indices)
    else:
        client.indices.flush(index=to_csv(indices), ignore_unavailable=True)
    try:
        client.indices.close(index=to_csv(indices), ignore_unavailable=True)
        return True
    except Exception:
        logger.error("Error closing indices.  Run with --debug flag and/or check Elasticsearch logs for more information.")
        return False

def close(client, indices):
    """
    Helper method called by the CLI.

    :arg client: The Elasticsearch client connection
    :arg indices: A list of indices to act on
    :rtype: bool
    """

    return close_indices(client, indices)



import click
from .index_selection import *

import logging
logger = logging.getLogger(__name__)

DEFAULT_ARGS = {
    'snapshot_prefix': 'curator-',
    'wait_for_completion': True,
    'include_global_state': True,
    'skip_repo_validation': False,
}

@cli.group('snapshot')
@click.option('--repository', help='Repository name.', expose_value=True)
@click.option('--name', help='Override default name.', expose_value=True)
@click.option('--prefix', help='Override default prefix.',
            expose_value=True, default=DEFAULT_ARGS['snapshot_prefix'])
@click.option('--wait_for_completion', type=bool, show_default=True,
            default=DEFAULT_ARGS['wait_for_completion'], expose_value=True,
            help='Wait for snapshot to complete before returning.')
@click.option('--ignore_unavailable', is_flag=True, expose_value=True,
            help='Ignore unavailable shards/indices.')
@click.option('--include_global_state', type=bool, show_default=True,
            default=DEFAULT_ARGS['include_global_state'], expose_value=True,
            help='Store cluster global state with snapshot.')
@click.option('--partial', is_flag=True, expose_value=True,
            help='Do not fail if primary shard is unavailable.')
@click.option('--request_timeout', type=int, default=21600, show_default=True,
            expose_value=True,
            help='Allow this many seconds before the transaction times out.')
@click.option('--skip-repo-validation', is_flag=True, expose_value=True,
            help='Skip repository access validation.')
@click.pass_context
def snapshot(
        ctx, repository, name, prefix, wait_for_completion, ignore_unavailable,
        include_global_state, partial, request_timeout, skip_repo_validation,
    ):
    """Take snapshots of indices (Backup)"""
    if not repository:
        click.echo('{0}'.format(ctx.get_help()))
        click.echo(click.style('Missing required parameter --repository', fg='red', bold=True))
        sys.exit(1)
snapshot.add_command(indices)



from .utils import *
import elasticsearch
import logging
logger = logging.getLogger(__name__)

def seal_indices(client, indices):
    """
    Seal the indicated indices (perform a synced flush) if your Elasticsearch
    version supports it before closing.
    This method will ignore unavailable (including closed) indices.

    :arg client: The Elasticsearch client connection
    :arg indices: A list of indices to act on
    :rtype: bool
    """
    indices = prune_closed(client, indices)
    es_version = get_version(client)
    errcode = ''
    results = {}
    try:
        if es_version >= (1, 6, 0):
            if len(indices) > 0:
                results = client.indices.flush_synced(index=to_csv(indices))
            else:
                logger.warn('No indices to seal.')
                return True
        else:
            logger.error('Your version of Elasticsearch ({0}) does not support index sealing (synced flush).  Must be 1.6.0 or higher.'.format(es_version))
    except Exception as e:
        logger.warn('Non-fatal error encountered.')
        try:
            logger.debug('Error: {0}.  Message: {1}'.format(e.status_code, e.error))
            results = e.info
        except AttributeError:
            logger.debug('Error: {0}'.format(e))
    total = results.pop('_shards') if '_shards' in results else {"total":0,"successful":0,"failed":0}
    fails = [ i for i in sorted(results) if results[i]['failed'] > 0 ]
    if len(fails) > 0:
        logger.warn('{0} indices failed to seal (synced flush):'.format(len(fails)))
        for f in fails:
            reasons = set()
            for shard in results[f]['failures']:
                reasons.add(shard['reason'])
            logger.error('{0}: {1}'.format(f, reasons))
            results.pop(f) # Remove fails from results
        if len(results) > 0:
            logger.info('All other indices provided have been successfully sealed. (Shown with --debug flag enabled.)')
    else:
        logger.info('Provided indices successfully sealed. (Shown with --debug flag enabled.)')
    if len(results) > 0:
        logger.debug('Successfully sealed indices: {0}'.format(list(results.keys())))
    return True # Failures are sometimes expected.  Log viewing is essential to ascertaining failures.

def seal(client, indices):
    """
    Helper method called by the CLI.

    :arg client: The Elasticsearch client connection
    :arg indices: A list of indices to act on
    :rtype: bool
    """

    return seal_indices(client, indices)



from .utils import *
import elasticsearch
import time
import logging
logger = logging.getLogger(__name__)

def optimize_index(client, index_name, max_num_segments=None, delay=0,
                request_timeout=21600):
    """
    Optimize (Lucene forceMerge) index to `max_num_segments` per shard

    :arg client: The Elasticsearch client connection
    :arg index_name: The index name
    :arg max_num_segments: Merge to this number of segments per shard.
    :arg delay: A number of seconds to delay after successfully optimizing
    :arg request_timeout: Number of seconds before the connection should timeout
    :rtype: bool
    """
    if optimized(client, index_name, max_num_segments):
        return True
    else:
        logger.info('Optimizing index {0} to {1} segments per shard.  Please wait...'.format(index_name, max_num_segments))
        try:
            client.indices.optimize(
                index=index_name, max_num_segments=max_num_segments,
                request_timeout=request_timeout
            )
            time.sleep(delay)
            return True
        except Exception:
            logger.error("Error optimizing index {0}.  Run with --debug flag and/or check Elasticsearch logs for more information.".format(index_name))
            return False

def optimize(client, indices, max_num_segments=None, delay=0, request_timeout=21600):
    """
    Helper method called by the CLI.

    :arg client: The Elasticsearch client connection
    :arg indices: A list of indices to act on
    :arg max_num_segments: Merge to this number of segments per shard.
    :arg delay: A number of seconds to delay after successfully optimizing
    :arg request_timeout: Number of seconds before the connection should timeout
    :rtype: bool
    """
    retval = True
    for i in ensure_list(indices):
        # If we fail once, we fail completely
        success = optimize_index(
                    client, i, max_num_segments=max_num_segments, delay=delay,
                    request_timeout=request_timeout
                  )
        if not success:
            retval = False
    return retval



import elasticsearch
import curator
import os
import click
from click import testing as clicktest
from mock import patch, Mock

from . import CuratorTestCase

import logging
logger = logging.getLogger(__name__)

host, port = os.environ.get('TEST_ES_SERVER', 'localhost:9200').split(':')
port = int(port) if port else 9200

class TestGetClient(CuratorTestCase):
    def test_get_client_positive(self):
        client_args = {"host":host, "port":port}
        client = curator.get_client(**client_args)
        self.assertTrue(isinstance(client, elasticsearch.client.Elasticsearch))
    def test_get_client_negative_connection_fail(self):
        client_args = {"host":host, "port":54321}
        with self.assertRaises(SystemExit) as cm:
            curator.get_client(**client_args)
        self.assertEqual(cm.exception.code, 1)

class TestCLIUtilsFilterCallback(CuratorTestCase):
    def test_filter_callback_without_timestring(self):
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'show',
                        'indices',
                        '--older-than', '5',
                        '--time-unit', 'days',
                    ],
                    obj={"filters":[]})
        self.assertEqual(1, result.exit_code)
    def test_filter_callback_without_timeunit(self):
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        'show',
                        'indices',
                        '--newer-than', '5',
                        '--timestring', '%Y.%m.%d',
                    ],
                    obj={"filters":[]})
        self.assertEqual(1, result.exit_code)

class TestSSLFlags(CuratorTestCase):
    def test_bad_certificate(self):
        self.create_indices(10)
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        '--use_ssl',
                        '--certificate', '/path/to/nowhere',
                        'show',
                        'indices',
                        '--older-than', '5',
                        '--time-unit', 'days',
                        '--timestring', '%Y.%m.%d',
                    ],
                    obj={"filters":[]})
        self.assertEqual(1, result.exit_code)
        self.assertEqual('Error: Could not open certificate at /path/to/nowhere\n', result.output)
    def test_ssl_no_validate(self):
        self.create_indices(10)
        test = clicktest.CliRunner()
        result = test.invoke(
                    curator.cli,
                    [
                        '--logfile', os.devnull,
                        '--host', host,
                        '--port', str(port),
                        '--use_ssl',
                        '--ssl-no-validate',
                        'show',
                        'indices',
                        '--older-than', '5',
                        '--time-unit', 'days',
                        '--timestring', '%Y.%m.%d',
                    ],
                    obj={"filters":[]})
        self.assertTrue(1, result.exit_code)



from .utils import *
import elasticsearch
import logging
logger = logging.getLogger(__name__)

def change_replicas(client, indices, replicas=None):
    """
    Change the number of replicas, more or less, for the indicated indices.

    :arg client: The Elasticsearch client connection
    :arg indices: A list of indices to act on
    :arg replicas: The number of replicas the indices should have
    :rtype: bool
    """
    if replicas == None:
        logger.error('No replica count provided.')
        return False
    else:
        indices = ensure_list(indices)
        pruned = prune_closed(client, indices)
        if sorted(indices) != sorted(pruned):
            logger.warn('Skipping closed indices to prevent error which can leave indices unopenable.')
        logger.info('Updating index setting: number_of_replicas={0}'.format(replicas))
        if len(pruned) > 0:
            try:
                client.indices.put_settings(index=to_csv(pruned),
                    body='number_of_replicas={0}'.format(replicas))
                return True
            except Exception:
                logger.error("Error changing replica count.  Run with --debug flag and/or check Elasticsearch logs for more information.")
                return False
        else:
            logger.warn('No open indices found.')

def replicas(client, indices, replicas=None):
    """
    Helper method called by the CLI.

    :arg client: The Elasticsearch client connection
    :arg indices: A list of indices to act on
    :arg replicas: The number of replicas the indices should have
    :rtype: bool
    """
    return change_replicas(client, indices, replicas=replicas)



from curator import api as curator

from mock import patch, Mock

from . import CuratorTestCase

class TestGetIndices(CuratorTestCase):
    def test_positive(self):
        self.create_index('test_index1')
        self.create_index('test_index2')
        l = sorted(curator.get_indices(self.client))
        r = ["test_index1", "test_index2"]
        self.assertEqual(r, l)
    def test_negative(self):
        l = sorted(curator.get_indices(self.client))
        r = ["test_index1", "test_index2"]
        self.assertNotEqual(r, l)
    def test_exception(self):
        client = "foo"
        l = curator.get_indices(client)
        self.assertFalse(l)

class TestCloseIndex1(CuratorTestCase):
    def test_positive(self):
        self.create_index('test_index')
        self.client.indices.close('test_index')
        self.assertTrue(curator.index_closed(self.client, 'test_index'))
    def test_negative(self):
        self.create_index('test_index')
        self.assertFalse(curator.index_closed(self.client, 'test_index'))

class TestCloseIndex2(CuratorTestCase):
    def test_index_will_be_closed(self):
        self.create_index('test_index')
        self.assertTrue(curator.close_indices(self.client, 'test_index'))
        index_metadata = self.client.cluster.state(
            index='test_index',
            metric='metadata',
        )
        self.assertEquals('close', index_metadata['metadata']['indices']['test_index']['state'])
    def test_closed_index_will_be_skipped(self):
        self.create_index('test_index')
        self.client.indices.close(index='test_index')
        self.assertTrue(curator.close_indices(self.client, 'test_index'))
        index_metadata = self.client.cluster.state(
            index='test_index',
            metric='metadata',
        )
        self.assertEquals('close', index_metadata['metadata']['indices']['test_index']['state'])

class TestPruneClosed(CuratorTestCase):
    def test_positive(self):
        for i in range(1, 11):
            self.create_index('test_index{0}'.format(i))
        self.client.indices.close(index='test_index1,test_index2,test_index3')
        l = curator.get_indices(self.client)
        l = sorted(curator.prune_closed(self.client, l))
        self.assertEqual(7, len(l))
        r = [
            "test_index10", "test_index4", "test_index5", "test_index6",
            "test_index7", "test_index8", "test_index9",
            ]
        self.assertEqual(l, r)
    def test_negative(self):
        for i in range(1, 11):
            self.create_index('test_index{0}'.format(i))
        self.client.indices.delete(index='test_index1,test_index2,test_index3')
        l = curator.get_indices(self.client)
        l = sorted(curator.prune_closed(self.client, l))
        self.assertEqual(7, len(l))
        r = [
            "test_index10", "test_index4", "test_index5", "test_index6",
            "test_index7", "test_index8", "test_index9",
            ]
        self.assertEqual(l, r)

class TestSegmentCount(CuratorTestCase):
    def test_simple(self):
        self.create_index('test_index', shards=2)
        self.client.index(index='test_index', doc_type='t', id=42, body={})
        self.client.indices.refresh(index='test_index')
        self.assertEquals((2, 1), curator.get_segmentcount(self.client, 'test_index'))



from . import *
from ..api import *
import elasticsearch
import click
import re

import logging
logger = logging.getLogger(__name__)


### SNAPSHOTS
@click.command(short_help="Snapshot selection.")
@click.option('--newer-than', type=int, callback=filter_callback,
                help='Include only snapshots newer than n time_units')
@click.option('--older-than', type=int, callback=filter_callback,
                help='Include only snapshots older than n time_units')
@click.option('--prefix', type=str, callback=filter_callback,
                help='Include only snapshots beginning with prefix.')
@click.option('--suffix', type=str, callback=filter_callback,
                help='Include only snapshots ending with suffix.')
@click.option('--time-unit', is_eager=True,
                type=click.Choice(['hours', 'days', 'weeks', 'months']),
                help='Unit of time to reckon by')
@click.option('--timestring', type=str, is_eager=True, default='%Y%m%d%H%M%S',
                help="Python strftime string to match your snapshot's definition, e.g. 20140715020304 would be %Y%m%d%H%M%S")
@click.option('--regex', type=str, callback=filter_callback,
                help="Provide your own regex, e.g '^prefix-.*-suffix$'")
@click.option('--exclude', multiple=True, callback=filter_callback,
                help='Exclude matching snapshots. Can be invoked multiple times.')
@click.option('--snapshot', multiple=True,
                help='Include the provided snapshot in the list. Can be invoked multiple times.')
@click.option('--all-snapshots', is_flag=True,
                help='Do not filter snapshots.  Act on all snapshots.')
@click.option('--repository', type=str, is_eager=True, expose_value=True,
                help='Repository name.')
@click.pass_context
def snapshots(ctx, newer_than, older_than, prefix, suffix, time_unit,
            timestring, regex, exclude, snapshot, all_snapshots, repository):
    """
    Get a list of snapshots to act on from the provided arguments, then perform
    the command [delete, show] on the resulting list.

    """

    if not repository:
        click.echo('{0}'.format(ctx.get_help()))
        click.echo(click.style('Missing required --repository parameter.', fg='red', bold=True))
        sys.exit(1)

    logger.info("Job starting: {0} snapshots".format(ctx.parent.info_name))

    # Base and client args are in the grandparent tier of the context
    if ctx.parent.parent.params['dry_run']:
        logging.info("DRY RUN MODE.  No changes will be made.")

    client = get_client(**ctx.parent.parent.params)
    # Get a master-list of indices
    snapshots = get_snapshots(client, repository=repository)
    if snapshots:
        working_list = snapshots
    else:
        click.echo(click.style('No snapshots found in Elasticsearch.', fg='red', bold=True))
        sys.exit(0)

    if all_snapshots:
        logger.info('Matching all snapshots. Ignoring flags other than --exclude.')
    else:
        logger.debug('All filters: {0}'.format(ctx.obj['filters']))

    for f in ctx.obj['filters']:
        if all_snapshots and not 'exclude' in f:
            continue
        logger.debug('Filter: {0}'.format(f))
        working_list = apply_filter(working_list, **f)

    # If there are manually added snapshots, we will add them here
    working_list.extend(in_list(snapshot, snapshots))

    if working_list:
        # Make a sorted, unique list of indices
        working_list = sorted(list(set(working_list)))
        logger.debug('ACTION: {0} will be executed against the following snapshots: {1}'.format(ctx.parent.info_name, working_list))
        if ctx.parent.info_name == 'show':
            logger.info('Matching snapshots:')
            show(client, working_list)
        elif ctx.parent.parent.params['dry_run']:
            show_dry_run(client, working_list, ctx.parent.info_name)
        elif ctx.parent.info_name == 'delete':
            success = True
            for snap in working_list:
                retval = delete_snapshot(client, snapshot=snap, repository=repository)
                # If we fail once, we fail completely
                if not retval:
                    success = False
            exit_msg(success)

    else:
        logger.warn('No snapshots matched provided args.')
        click.echo(click.style('No snapshots matched provided args.', fg='red', bold=True))
        sys.exit(0)



from curator import api as curator

from mock import patch, Mock

from . import CuratorTestCase

class TestAlias(CuratorTestCase):
    def test_add_to_pre_existent_alias(self):
        alias = 'testalias'
        self.create_index('dummy')
        self.client.indices.put_alias(index='dummy', name=alias)
        self.create_index('foo')
        curator.add_to_alias(self.client, 'foo', alias=alias)
        self.assertEquals(2, len(self.client.indices.get_alias(name=alias)))
    def test_add_to_non_existent_alias(self):
        alias = 'testalias'
        self.create_index('dummy')
        self.client.indices.put_alias(index='dummy', name=alias)
        self.create_index('foo')
        self.assertTrue(curator.add_to_alias(self.client, 'foo', alias="ooga"))
    def test_add_to_alias_with_closed(self):
        alias = 'testalias'
        self.create_index('dummy')
        self.client.indices.put_alias(index='dummy', name=alias)
        self.create_index('foo')
        curator.close_indices(self.client, 'foo')
        self.assertTrue(curator.index_closed(self.client, 'foo'))
        curator.add_to_alias(self.client, 'foo', alias=alias)
        self.assertFalse(curator.add_to_alias(self.client, 'foo', alias=alias))
    def test_add_to_alias_idx_already_in_alias(self):
        alias = 'testalias'
        self.create_index('dummy')
        self.client.indices.put_alias(index='dummy', name=alias)
        self.create_index('foo')
        curator.add_to_alias(self.client, 'foo', alias=alias)
        self.assertTrue(curator.add_to_alias(self.client, 'foo', alias=alias))
    def test_remove_from_alias_positive(self):
        alias = 'testalias'
        self.create_index('dummy')
        self.client.indices.put_alias(index='dummy', name=alias)
        self.create_index('foo')
        curator.add_to_alias(self.client, 'foo', alias=alias)
        curator.remove_from_alias(self.client, 'dummy', alias=alias)
        self.assertEquals(1, len(self.client.indices.get_alias(name=alias)))
    def test_remove_from_alias_negative(self):
        alias = 'testalias'
        self.create_index('dummy')
        self.client.indices.put_alias(index='dummy', name=alias)
        self.assertFalse(curator.remove_from_alias(self.client, 'dummy', alias="ooga"))
    def test_full_alias_add_to_pre_existent_alias(self):
        alias = 'testalias'
        self.create_index('dummy')
        self.client.indices.put_alias(index='dummy', name=alias)
        self.create_index('foo')
        self.assertTrue(curator.alias(self.client, 'foo', alias=alias))
        self.assertEquals(2, len(self.client.indices.get_alias(name=alias)))
    def test_full_alias_add_non_existent_index(self):
        alias = 'testalias'
        self.create_index('dummy')
        self.client.indices.put_alias(index='dummy', name=alias)
        self.assertFalse(curator.alias(self.client, 'foo', alias='ooga'))
    def test_full_alias_remove_positive(self):
        alias = 'testalias'
        self.create_index('dummy')
        self.client.indices.put_alias(index='dummy', name=alias)
        self.create_index('foo')
        curator.add_to_alias(self.client, 'foo', alias=alias)
        curator.alias(self.client, 'dummy', alias=alias, remove=True)
        self.assertEquals(1, len(self.client.indices.get_alias(name=alias)))
    def test_full_alias_remove_negative(self):
        alias = 'testalias'
        self.create_index('dummy')
        self.client.indices.put_alias(index='dummy', name=alias)
        self.assertFalse(curator.alias(self.client, 'dummy', alias="ooga", remove=True))

class TestChangeReplicas(CuratorTestCase):
    def test_index_replicas_can_be_modified(self):
        self.create_index('test_index')
        self.assertEquals('0', self.client.indices.get_settings(index='test_index')['test_index']['settings']['index']['number_of_replicas'])

        curator.change_replicas(self.client, 'test_index', replicas=1)

        self.assertEquals('1', self.client.indices.get_settings(index='test_index')['test_index']['settings']['index']['number_of_replicas'])

    def test_index_replicas_untouched(self):
        self.create_index('test_index')
        self.assertEquals('0', self.client.indices.get_settings(index='test_index')['test_index']['settings']['index']['number_of_replicas'])

        curator.change_replicas(self.client, 'test_index', replicas=0)

        self.assertEquals('0', self.client.indices.get_settings(index='test_index')['test_index']['settings']['index']['number_of_replicas'])

    def test_closed_index_replicas_cannot_be_modified(self):
        self.create_index('test_index')

        self.client.indices.close(index='test_index')
        index_metadata = self.client.cluster.state(
            index='test_index',
            metric='metadata',
        )
        self.assertEquals('close', index_metadata['metadata']['indices']['test_index']['state'])

        self.assertEquals('0', self.client.indices.get_settings(index='test_index')['test_index']['settings']['index']['number_of_replicas'])

        curator.change_replicas(self.client, 'test_index', replicas=1)

        self.assertEquals('0', self.client.indices.get_settings(index='test_index')['test_index']['settings']['index']['number_of_replicas'])

        self.client.indices.open(index='test_index')
        index_metadata = self.client.cluster.state(
            index='test_index',
            metric='metadata',
        )
        self.assertEquals('open', index_metadata['metadata']['indices']['test_index']['state'])

        self.assertEquals('0', self.client.indices.get_settings(index='test_index')['test_index']['settings']['index']['number_of_replicas'])

class TestChangeAllocation(CuratorTestCase):
    def test_index_allocation_can_be_modified(self):
        self.create_index('test_index')

        curator.apply_allocation_rule(self.client, 'test_index', rule="key=value")

        self.assertEquals('value', self.client.indices.get_settings(index='test_index')['test_index']['settings']['index']['routing']['allocation']['require']['key'])

    def test_index_allocation_can_be_modified_for_include(self):
        self.create_index('test_index')

        curator.apply_allocation_rule(self.client, 'test_index', rule="key=value", allocation_type='include')

        self.assertEquals('value', self.client.indices.get_settings(index='test_index')['test_index']['settings']['index']['routing']['allocation']['include']['key'])

    def test_index_allocation_can_be_modified_for_exclude(self):
        self.create_index('test_index')

        curator.apply_allocation_rule(self.client, 'test_index', rule="key=value", allocation_type='exclude')

        self.assertEquals('value', self.client.indices.get_settings(index='test_index')['test_index']['settings']['index']['routing']['allocation']['exclude']['key'])

    def test_closed_index_allocation_can_be_modified(self):
        self.create_index('test_index')

        self.client.indices.close(index='test_index')
        index_metadata = self.client.cluster.state(
            index='test_index',
            metric='metadata',
        )
        self.assertEquals('close', index_metadata['metadata']['indices']['test_index']['state'])

        curator.apply_allocation_rule(self.client, 'test_index', rule="key=value")

        self.assertEquals('value', self.client.indices.get_settings(index='test_index')['test_index']['settings']['index']['routing']['allocation']['require']['key'])

        self.client.indices.open(index='test_index')
        index_metadata = self.client.cluster.state(
            index='test_index',
            metric='metadata',
        )
        self.assertEquals('open', index_metadata['metadata']['indices']['test_index']['state'])

        self.assertEquals('value', self.client.indices.get_settings(index='test_index')['test_index']['settings']['index']['routing']['allocation']['require']['key'])


class TestDeleteIndex(CuratorTestCase):
    def test_index_will_be_deleted(self):
        self.create_index('test_index')
        self.assertTrue(curator.delete_indices(self.client, 'test_index'))
        self.assertFalse(self.client.indices.exists('test_index'))

class TestBloomIndex(CuratorTestCase):
    def test_bloom_filter_will_be_disabled(self):
        self.create_index('test_index')
        # Bloom filters have been removed from the 1.x branch after 1.4.0
        no_more_bloom = (1, 4, 0)
        version_number = curator.get_version(self.client)
        if version_number < no_more_bloom:
            self.assertTrue(curator.disable_bloom_filter(self.client, 'test_index'))
            settings = self.client.indices.get_settings(index='test_index')
            self.assertEquals('false', settings['test_index']['settings']['index']['codec']['bloom']['load'])

    def test_closed_index_will_be_skipped(self):
        self.create_index('test_index')
        self.client.indices.close(index='test_index')
        self.assertTrue(curator.disable_bloom_filter(self.client, 'test_index'))
        index_metadata = self.client.cluster.state(
            index='test_index',
            metric='metadata',
        )
        self.assertEquals('close', index_metadata['metadata']['indices']['test_index']['state'])

class TestOptimizeIndex(CuratorTestCase):
    def test_optimized_index_will_be_skipped(self):
        self.create_index('test_index')
        self.client.create(index='test_index', doc_type='log', body={'message':'TEST DOCUMENT'})
        # Will only have 1 segment
        self.assertTrue(curator.optimize_index(self.client, 'test_index', max_num_segments=4))



import click
from .index_selection import *

import logging
logger = logging.getLogger(__name__)

@cli.group('seal')
@click.pass_context
def seal(ctx):
    """
    Seal indices
    (Synced flush: ES 1.6.0+ only)
    """
seal.add_command(indices)
